import org.flowplayer.ClipDescriptor;
import org.flowlib.NewAbstractMovieClip;
/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
class org.flowlib.AbstractScrollBar extends NewAbstractMovieClip {
	
	private var btnScroll:NewAbstractMovieClip;
	private var btnUpScroll:MovieClip;
	private var btnDownScroll:MovieClip;
	private var mcBorder:MovieClip;
	private var scrollableRatio:Number;
	private var eventFiringTreshold:Number = 0;
	private var interevalId:Number;
	
	private var prevScrollValue:Number;
	private var scrollPosition:Number;
	private var gripPos:Number;
	private var scrollLength:Number;
		
	private function getChildDescriptors():Array {
		return [ getUpButtonDescriptor(), getDownButtonDescriptor(), getHandleDescriptor() ];
	}
	
	private function allChildrenCreated(clips:Array):Void {
		btnUpScroll = clips[0];
		btnDownScroll = clips[1];
		
		btnUpScroll.onPress = function():Void {
			this._parent.decrease();
		};
		
		btnDownScroll.onPress = function():Void {
			this._parent.increase();
		};

		btnScroll = clips[2];

		btnScroll.onPress = function():Void {
			this._parent.gripPos = this._parent.getMousePosition() - this._parent.getBtnSrollPosition();
			this._parent.interevalId = setInterval(this._parent, "scroll", 10);
		};
		btnScroll.onRelease = function():Void {
			clearInterval(this._parent.interevalId);
			this._parent.dispatchScrollEvent();
		};
		btnScroll.onReleaseOutside = btnScroll.onRelease;
	}
	
	private function getUpButtonDescriptor():ClipDescriptor {
		throw new Error("this should be overridden");
		return null;
	}
	
	private function getDownButtonDescriptor():ClipDescriptor {
		throw new Error("this should be overridden");
		return null;
	}
	
	private function getHandleDescriptor():ClipDescriptor {
		throw new Error("this should be overridden");
		return null;
	}
	
	public function setScrollLengths(totalLength:Number, visibleLength:Number, height:Number):Void {
		scrollableRatio = totalLength / visibleLength;
		setSize(totalLength, height, true);
	}
	
	public function setEventFiringTreshold(value:Number):Void {
		this.eventFiringTreshold = value;
	}
	
	private function createChildren():Void {
		logger.debug("Creating additional children");
		mcBorder = createEmptyMovieClip("mcBorder", getNextHighestDepth());
	}
	
	private function getBtnSrollPosition() : Number {
		return getPos(btnScroll);
	}
	
	private function arrange():Void {
		logger.debug("AbstractScrollBar(" +this._name + ").arrange() !! ");
		if (! allChildrenLoaded()) return;
		if (! __height > 0) return;
		
		logger.debug(getDimensions(this));
		logger.debug(getDimensions(btnUpScroll));
		logger.debug(getDimensions(btnDownScroll));
		
		btnUpScroll._x = 0;
		btnUpScroll._y = 0;

		setPos(btnScroll, getLength(btnUpScroll));
		setPos(btnDownScroll, getLength(this) - getLength(btnDownScroll));
		scrollLength = getLength(this) - getLength(btnUpScroll) - getLength(btnDownScroll);
		setFixedLength();

		mcBorder.clear();
		mcBorder.beginFill(0xDDDDDD, 100);
		drawRectangle(mcBorder, 0, 0, __width, __height, false);
		mcBorder.endFill();
		mcBorder._x = 0;
		mcBorder._y = 0;
		mcBorder._width = __width;
		mcBorder._height = __height;
		
		logger.debug("scrollLength = " + scrollLength + ", scrollable ratio = " + scrollableRatio);
		
		if (scrollableRatio < 1) {
			setLength(btnScroll, scrollLength * scrollableRatio, true);
		}
		
		logger.info(getDimensions(this));
		logger.info(getDimensions(btnScroll));
	}
	
	private function scroll():Void {
		var newValue:Number = getMousePosition() - gripPos;
		
		scrollTo(newValue);

		if (requiresEvent(newValue, prevScrollValue)) {
			dispatchScrollEvent();
		}	
		prevScrollValue = newValue;
	}
	
	private function scrollTo(newValue:Number):Void {
		var maxDragValue:Number = maxDragValue();
		if (newValue > maxDragValue)
			newValue = maxDragValue;
		else {
			var minDragValue:Number = minDragValue();
			if (newValue < minDragValue)
				newValue = minDragValue;
		}
		setPos(btnScroll, newValue);
	}
	
	private function maxDragValue():Number {
		return getLength(this) - getLength(btnScroll) - getLength(btnDownScroll);
	}
	
	private function minDragValue():Number {
		return getLength(btnUpScroll);
	}
	
	private function requiresEvent(newValue:Number, oldValue:Number):Boolean {
		return newValue > oldValue ? newValue - oldValue > eventFiringTreshold : oldValue - newValue > eventFiringTreshold;
	}
	
	private function dispatchScrollEvent():Void {
		scrollPosition = calculateScrollPosition();
		dispatchEvent("onScroll", this);
		prevScrollValue = getLength(btnScroll);
	}
	
	private function calculateScrollPosition():Number {
		var value:Number = (getPos(btnScroll) - getLength(btnUpScroll)) / (maxDragValue() - getLength(btnUpScroll));
//		logger.debug("calculated scroll value " + value);
//		logger.debug("pos " + getPos(btnScroll));
//		logger.debug(getLength(btnUpScroll));
//		logger.debug(maxDragValue());
		return value;
	}
	
	public function getScrollPosition():Number {
		return this.scrollPosition;
	}
	
	public function setScrollPosition(scrollRatio:Number):Void {
		scrollTo(scrollRatio * maxDragValue());
	}
	
	private function decrease():Void {
		scrollTo(getPos(btnScroll) - 10);
		dispatchScrollEvent();
	}
	
	private function increase():Void {
		scrollTo(getPos(btnScroll) + 10);
		dispatchScrollEvent();
	}
	
	/*
	 *Abstract functions to be overridden in subclasses:
	 *--------------------------------------------------
	 */
	
	private function getMousePosition() : Number {
		return null;
	}
	
	private function setLength(clip:MovieClip, length:Number):Void {
	}
	
	private function getLength(clip:MovieClip):Number {
		return null;
	}
	
	private function getPos(clip:MovieClip):Number {
		return null;
	}
	
	private function setPos(clip:MovieClip, pos:Number):Void {
	}
	
	private function setFixedLength():Void {
	}
}