/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.NewAbstractMovieClip;
// TODO: move following classes to flowlib
import org.flowplayer.FlowText;
import org.flowplayer.ClipDescriptor;
import org.flowplayer.Util;
 
class org.flowlib.AbstractSlider extends NewAbstractMovieClip {
 
	public static var BAR_TO_RIGHT_EDGE_GAP_DEFAULT:Number = 0;
	
	// following are public so that they can be arranged in skins
	public static var LABEL_HEIGHT:Number = 15;
  	public var mcValue:MovieClip;
	public var mcBackground:MovieClip;
  	public var mcBorder:MovieClip;
  	public var mcDragger:MovieClip;
  	public var mcValue2:MovieClip;
	public var mcLabel : MovieClip;
	public var dragClip : MovieClip;
	
  	private var progressPercent:Number;
  	private var dragOffset:Number;
	private var prevDragValue:Number;
	private var gripPos:Number = 0;
	
    // The interval identifiers used with setInterval() and clearInterval()
  	private var intervalId:Number;
  	private var dragEventDispatchIntervalId: Number;

	private function createChildren():Void {
		createBackground();
		createBorder();
		createValueIndicator2();
		createValueIndicator();
		createTextLabel();
		registerDragEvent();
	}

	// Arrange all the elements. This method gets called when the instance is first created as well
	// as every time it is resized.
	 function arrange():Void {
		arrangeSlider(this);
		onArranged();
	}

	public function get barXPos():Number {
		// possibly overridden by subclasses
		return 0;
	}
	
	public function get barToRightEdgeGap():Number {
		return BAR_TO_RIGHT_EDGE_GAP_DEFAULT;
	}
	
	public function get edgeClipsHeight():Number {
		// possibly overridden by subclasses
		return __height;
	}
	
	public function get barHeight():Number {
		// possibly overridden by subclasses
		return __height;
	}
	
	public function get progressBarWidth():Number {
		// possibly overridden by subclasses
		return mcBorder._width;
	}
	
	private function onArranged():Void {
		// possibly overridden by subclasses
	}
	
	public function removeLabel():Void {
		mcLabel.removeMovieClip();
	}

	public function setX(newX:Number):Void {
		trace("tracker: setx to " + newX);
		_x = newX;
	}

	public function setY(newY:Number):Void {
		_y = newY;
	}
	
	// TODO: Complex calculations in this class should be refactored and encapsulated (more object oriented)
	public function getSeekPosition():Number {
		var value:Number = ((mcDragger._x + mcDragger._width/2 - barXPos) / mcBorder._width) * 100;
		if (value > 100)
			value = 100;
		logger.debug("calculated seek position " + value);
		return value;
	}
	
	public function addSeekListener(listener:Object):Void {
		trace("adding 'onSeek' listener " + listener);
		addEventListener("onSeek", listener);
	}
	
	private function createValueIndicator() {
		mcValue = createEmptyMovieClip("mcValue", getNextHighestDepth());
		onCreateValueIndicator(mcValue);
	}
		
	private function createValueIndicator2() {
		mcValue2 = createEmptyMovieClip("mcValue2", getNextHighestDepth());
		mcValue2._width = 0;
		onCreateValueIndicator2(mcValue2);
	}

	private function onCreateValueIndicator(mcValue:MovieClip):Void {
		// possibly overridden by subclasses
	}

	private function onCreateValueIndicator2(mcValue:MovieClip):Void {
		// possibly overridden by subclasses
	}
	
	
	private function createTextLabel() : Void {
		mcLabel = createEmptyMovieClip("mcLabel", getNextHighestDepth());
		mcLabel.beginFill(0, 34);
		Util.drawRoundedRectacle(mcLabel, 0, 0, 70, LABEL_HEIGHT, 8);
		mcLabel.endFill();
		
		var text:TextField = Util.createText(mcLabel, getSkin(), "text", "", false, false, false, 70, 5, getSkin().getTimeDisplayFontSize(), "Sans");
		text.autoSize = true;
		setLabelText("00:00 / 00:00");
	}
	
	private function setLabelText(text:String) : Void {
	}
	
	private function createBackground() {
		createEmptyMovieClip("mcBackground", getNextHighestDepth());
		onCreateBackground(mcBackground);
	}
	
	private function createBorder() {
		createEmptyMovieClip("mcBorder", getNextHighestDepth());
		onCreateBorder(mcBorder);
	}
	
	private function loadImage(parent:MovieClip, clipName:String, fileURL:String):MovieClip {
		NewAbstractMovieClip.logger.info("Loading image " + fileURL + ", clipName = " + clipName);
		var clip:MovieClip = parent.createEmptyMovieClip(clipName, getNextHighestDepth());
		loader.loadClip(fileURL, clip);
		return clip;
	}
	
	private function onCreateBorder(mcBorder:MovieClip):Void {
		// possibly overridden in subclasses
	}
	
	private function onCreateBackground(mcBackground:MovieClip):Void {
		// possibly overridden in subclasses
	}
	
	public function registerDragEvent():Void {
		dragClip = allowSeekingBeyondValue2() ? mcBorder : mcValue2;
		
		dragClip.onRelease = function():Void {
			if (! this.enabled) return;
			if (! this._parent.isMouseOverClip(this)) return;
			this._parent.drag();
			this._parent.dispatchDragEvent();
		};
	}
	
	private function allowSeekingBeyondValue2():Boolean {
		// possibly overridden in subclasses
		return true;
	}

	private function getChildDescriptors():Array {
		// overridden in subclasses
		return null;
	}

	private function arrangeSlider(mcSlider:AbstractSlider):Void {
		// overridden in subclasses
	}
	
	private function allChildrenCreated(clips:Array):Void {
		mcDragger = clips[0];
		mcDragger.onPress = function():Void {
			this._parent.gripPos = this._xmouse;
			this._parent.interevalId = setInterval(this._parent, "drag", 10);
		};
		mcDragger.onRelease = function():Void {
			clearInterval(this._parent.interevalId);
			this._parent.dispatchDragEvent();
		};
		mcDragger.onReleaseOutside = mcDragger.onRelease;
		
		onAllChildrenCreated();
	}
	
	private function onAllChildrenCreated(clips:Array):Void {
		// possibly overridden in subclasses
	}
		
	private function drag():Void {
		setDraggerPos(_xmouse);
		//updateAfterEvent();
		if (getConfig().getStreamingServer() != "lighttpd")
			dispatchDragEvent();
	}
	
	public function setValue(ratio:Number, setSecondValue:Boolean) {
		if (! ratio || ratio < 0)
			ratio = 0;
		if (ratio > 1)
			ratio = 1;
		
		var value:Number = seekPosAtEnd  * ratio;
		if (setSecondValue)
			mcValue2._width = value;
		else {
			mcValue._width = value;
			setDraggerPos(mcValue._x + mcValue._width);
		}
	}
	
	private function setDraggerPos(pos:Number):Void {
		pos = pos - gripPos;
		
		if (pos <= minSeekPosNow) { 
			pos = minSeekPosNow;
		} else if (pos >= maxSeekPosNow) {
			pos = maxSeekPosNow;
		}
		if (! pos) return;
		mcDragger._x = pos;		
	}

	private function get minSeekPosNow():Number {
		return mcBorder._x + 1;
	}

	private function get maxSeekPosNow():Number {
		var rightEdge:Number = allowSeekingBeyondValue2() ? seekPosAtEnd : mcValue2._x + mcValue2._width;
		if (rightEdge - mcDragger._width - 1 < minSeekPosNow) return minSeekPosNow;
		return rightEdge - mcDragger._width - 1;
	}

	private function get seekPosAtEnd():Number {
		return mcBorder._x + mcBorder._width - barXPos + barToRightEdgeGap;		
	}

	private function dispatchDragEvent():Void {
		if(mcDragger._x != prevDragValue) {
			dispatchEvent("onSeek", this);
			prevDragValue = mcDragger._x;
		}
	}
	
	public function set enabled(makeEnabled:Boolean):Void {
		super.enabled = makeEnabled;
		dragClip.enabled = makeEnabled;
		mcBorder.enabled = makeEnabled;
		mcDragger.enabled = makeEnabled;
		mcValue.enabled = makeEnabled;
		mcValue2.enabled = makeEnabled;
	}

}
