import LuminicBox.Log.Logger;
import LuminicBox.Log.ConsolePublisher;
/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author Anssi
 */
class org.flowlib.EaseMove {

	private var easeId:Number;
	private var logger:Logger;
	private var delayIntervalId:Number;
	
	private function EaseMove() {
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
		delayIntervalId = -1;
	}

	public static function execute(clip:MovieClip, targetPos:Number):Void {
		var move:EaseMove = new EaseMove();
//		stopIfActive(clip);
		move.startEase(clip, targetPos, 0.2);
	}
	
	public static function stopIfActive(clip:MovieClip):Void {
		stopEffect(clip, "delayedEaseEffect");
		stopEffect(clip, "easeEffect");
	}
	
	private static function stopEffect(clip:MovieClip, effectName:String):Void {
		var effect:EaseMove = clip[effectName];
		effect.stop();
		clip[effectName] = undefined;
	}

	public static function executeDelayed(clip:MovieClip, delayMillis:Number, targetPos:Number):Void {
		if (clip._y == targetPos) return;
//		stopIfActive(clip);
		var move:EaseMove = new EaseMove();
		move.delayedEase(clip, delayMillis, targetPos);		
	}
	
	public function stop():Void {
		clearInterval(delayIntervalId);
		delayIntervalId = -1;
		clearInterval(easeId);
		easeId = -1;
	}
	
	private function delayedEase(clip:MovieClip, delayMillis:Number, targetPos:Number):Void {
		clip.delayedEaseEffect = this;
		delayIntervalId = setInterval(this, "startEase", delayMillis, clip, targetPos, 0.2);
	}

	private function startEase(clip:MovieClip, targetPos:Number, easing:Number):Void {
		if (clip._y == targetPos) return;
		if (clip.easeEffect) return;
		stop();
		clip.easeEffect = this;
		logger.debug("starting ease, targetPos " + targetPos);
		easeId = setInterval(this, "ease", 10, clip, targetPos, easing);
	}

	private function ease(clip:MovieClip, targetPos:Number, easing:Number):Void {
		var inTarget:Boolean = moveClip(clip, targetPos, easing);

		if (inTarget) {
			clearInterval(easeId);
			clip.easeEffect = undefined;
			easeId = -1;
			logger.debug("clips easing done, clearing interval");
			
		}
	}

	private function moveClip(clip:MovieClip, targetPos:Number, easing:Number):Boolean {
		var vy:Number = (targetPos - clip._y) * easing;
		clip._y += vy;

		var dy:Number = targetPos - clip._y;
		logger.debug("dy " + dy);
		if (Math.abs(dy) < 1) {
			clip._y = targetPos;
			return true;
		}
		return false;
	}

}