/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.as2lib.data.holder.Map;
import org.as2lib.data.holder.map.HashMap;
import org.as2lib.env.event.multicaster.EventMulticaster;
import org.as2lib.env.event.multicaster.SimpleEventMulticaster;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;

/**
 * EventDispatcher holds a group of event listeners and is able
 * to dispatch events to them.
 */
class org.flowlib.EventDispatcher extends MovieClip {
	private var multicasters:Map;
	private var logger:Logger;
	
	public function EventDispatcher() {
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
		logger.info("EventDispatcher::Constructor");
		multicasters = new HashMap();
	}
	
	private function getMultiCaster(typeName:String):EventMulticaster {
		//logger.debug("multicasters map: " + multicasters);
		return multicasters.get(typeName);
	}
				
	/**
	 * Adds a listener to the specified event
	 *
	 * @param typenName name that identifies the event to listen
	 * @param listener
	 */
	public function addEventListener(typeName:String, listener:Object):Void {
		var multiCaster:EventMulticaster = getMultiCaster(typeName);
		if (multiCaster == null) {
			multiCaster = new SimpleEventMulticaster(typeName);
			multicasters.put(typeName, multiCaster);
		} else {
		}
		multiCaster.addListener(listener);
	}
	
	
	/**
	 * Removes a registered event listener.
	 */
	public function removeListener(typeName:String, listener:Object):Void {
		var multiCaster:EventMulticaster = getMultiCaster(typeName);
		if (multiCaster) {
			multiCaster.removeListener(listener);
		}
	}
			
	/**
	 * Dispatches an event to all registered listeners.
	 *
	 * @param typenName name that identifies the event to dispatch
	 * @param eventObject an object to be passed over to the listener
	 */
	public function dispatchEvent(typeName:String, eventObject:Object):Void {
		//logger.info(this + ":: dispatching " + typeName + ", multicaster =  " + getMultiCaster(typeName));
		if (eventObject != null)
			getMultiCaster(typeName).dispatch(eventObject);
		else
			getMultiCaster(typeName).dispatch();
	}
	
}