﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.ClipDescriptor;

/**
 * An abstract ToggleButton.
 */
class org.flowplayer.AbstractToggleButton extends NewAbstractMovieClip {
	
	private var mcDown:MovieClip;
	private var mcUp:MovieClip;
	private var isDown:Boolean;
	
	private function getChildDescriptors():Array {
		return [
			new ClipDescriptor(getDownMcSymbolName(), "mcDown", null),
			new ClipDescriptor(getUpMcSymbolName(), "mcUp", null)
		];
	}
	
	private function allChildrenCreated(clips:Array):Void {
		mcDown = clips[0];
		mcUp = clips[1];
		toggled.button = this;
		this.onRelease = toggled;
		
		this.onRollOver = function():Void {
			if (this.isDown)
				this.mcDown.onRollOver();
			else
				this.mcUp.onRollOver();
		};
		this.onRollOut = function():Void {
			if (this.isDown)
				this.mcDown.onRollOut();
			else
				this.mcUp.onRollOut();
		};
	}
	
	public function initState(isDown:Boolean):Void {
		this.isDown = isDown;
		changeClips();
	}
	
	public function arrange():Void {
		mcDown._x = 0;
		mcUp._x = 0;
		mcDown._y = 0;
		mcUp.y = 0;
		mcDown._width = this._width;
		mcUp._width = this._width;
		mcDown._height = this._height;
		mcUp._height = this._height;
		changeClips();
	}
	
	/**
	 * Override this function if "down image" should be attached from library.
	 */
	public function getDownMcSymbolName():String {
		return null;
	}

	/**
	 * Override this function if "up image" should be attached from library.
	 */
	public function getUpMcSymbolName():String {
		return null;
	}
	
	public function toggled():Void {
		var button:AbstractToggleButton = arguments.callee.button;
		if (button.isDownPressed())
			button.release();
		else
			button.pressDown();
		dispatchEvent("onToggled", arguments.callee.button);
	}
	
	public function pressDown():Void {
		this.isDown = true;
		changeClips();
	}
	
	public function release():Void {
		this.isDown = false;
		changeClips();
	}
	
	public function isDownPressed():Boolean {
		return isDown;
	}
	
	private function changeClips():Void {
		mcDown._visible = this.isDown;
		mcUp._visible = ! this.isDown;
	}	
}