import org.flowlib.ColorFader;
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.AbstractToggleButton;
import org.flowplayer.ClipDescriptor;
import org.flowplayer.FLVDisplay;
import org.flowplayer.FlowPlayerExternalInterface;
import org.flowplayer.MediaControl;
import org.flowplayer.PlaybackStatus;
import org.flowplayer.ProgressTracker;
import org.flowplayer.Util;
import org.flowplayer.VolumeSlider;
import org.flowplayer.button.FlowButton;
import org.flowplayer.config.Config;
import org.flowplayer.skin.Skin;
import org.flowplayer.button.MuteVolumeToggleButton;

import flash.geom.Rectangle;

/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
class org.flowplayer.ControlBar extends NewAbstractMovieClip {
	
	public var mcBackground:MovieClip;
	public var stopButton:MovieClip;
	public var playButton:MovieClip;
	public var pauseButton:MovieClip;
	public var nextButton:MovieClip;
	public var prevButton:MovieClip;
	public var tracker:ProgressTracker;
	public var volume:VolumeSlider;
	public var menuButton:MovieClip;
	public var fullScreenButton:MovieClip;
	public var muteVolumeButton:MuteVolumeToggleButton;

	private var maxControlHeight:Number;
	private var fader:ColorFader;
	private var control:MediaControl;
	var mySkin:Skin;
	var trackIntervalId:Number = -1;
	var bufferingIntervalId:Number = -1;
	private var display:FLVDisplay;
	private var originalWidth:Number;

	public function setControl(control:MediaControl):Void {
		this.control = control;
		this.play.control = this.control;
		this.pause.control = this.control;
		this.updateProgress.control = this.control;
		this.onToggled.control = this.control;
		this.onToggled.bar = this;
		createTrackUpdateInterval();
		
		control.addListener("onPlay", this);
		control.addListener("onStop", this);
		control.addListener("onPause", this);
		control.addListener("onResume", this);
		volume.addVolumeEventListener(this);
		this.addEventListener("onVolumeChanged", control);
	}

	public function setDisplay(display:FLVDisplay):Void {
		this.onToggled.display = display;
		this.display = display;
	}

	private function getChildDescriptors():Array {
		var descriptors:Array = new Array();
		
		if (showMuteVolumeButton())
			descriptors.push(new ClipDescriptor("MuteVolumeToggleButton", "muteVolumeButton"));
		if (showVolumeSlider())
			descriptors.push(new ClipDescriptor("VolumeControl", "volume", null));
		if (showScrubber())
			descriptors.push(new ClipDescriptor("FlowTracker", "tracker", null));
		if (showPlayButton())
			descriptors.push(new ClipDescriptor("PlayButton", "playButton", null));
		if (showPauseButton())
			descriptors.push(new ClipDescriptor("PauseButton", "pauseButton", null));
		if (showStopButton())
			descriptors.push(new ClipDescriptor("StopButton", "stopButton", null));
		
		// full screen button is always present, might be just hidden
		// we need it to toggle full-screen
		descriptors.push(new ClipDescriptor("ToggleFullScreenButton", "fullScreenButton", null));
		
		if (usePlayList()) {
			descriptors.push(new ClipDescriptor("NextButton", "nextButton"));
			descriptors.push(new ClipDescriptor("PrevButton", "prevButton"));
		}
		if (getConfig().showMenu()) {
			descriptors.push(new ClipDescriptor("MenuButton", "menuButton"));
		}
		
		logger.debug("descriptors:");
		logger.debug(descriptors);
		return descriptors;
	}
	
	public function showScrubber():Boolean {
		return getConfig().showScrubber();
	}

	private function allChildrenCreated(clips:Array):Void {
		fader = new ColorFader();
		addClipsToFader(clips);

		var i:Number = 0;
		
		if (showMuteVolumeButton()) {
			muteVolumeButton = clips[i++];
			muteVolumeButton.initState(false);
			muteVolumeButton.addEventListener("onToggled", this);
		}
		if (showVolumeSlider()) {
			volume = clips[i++];
			volume.addVolumeEventListener(this);
		}
		if (showScrubber()) {
			tracker = clips[i++];
			tracker.addSeekListener(this);
			tracker.addProgressListener(getSkin());
		}
		if (showPlayButton()) {		
			playButton = clips[i++];
			playButton.onRelease = function():Void { 
				this._parent.play(); 
			};
		}
		if (showPauseButton()) {
			pauseButton = clips[i++];
			pauseButton.onRelease = function():Void {
				this._parent.pause(); 
			};
		}
		if (showStopButton()) {
			stopButton = clips[i++];
			stopButton.onRelease = function():Void {
				this._parent.rewind();
			};
			stopButton.enabled = false;
		}

		fullScreenButton = clips[i++];
		fullScreenButton.initState(getConfig().isFullScreen());
		fullScreenButton.addEventListener("onToggled", this);
		if (! showFullScreenButton())
			fullScreenButton._visible = false;
		if (getConfig().useHwScaling())
			fullScreenButton.enabled = false;

		if (usePlayList()) {
			nextButton = clips[i++];
			nextButton.onRelease = function():Void {
				NewAbstractMovieClip(this._parent).getConfig().getPlayList().next();
			};
			prevButton = clips[i++];
			prevButton.onRelease = function():Void {
				NewAbstractMovieClip(this._parent).getConfig().getPlayList().prev();
			};
		}
		if (showMenu()) {
			menuButton = clips[i++];
			menuButton.onRelease = function():Void {
				this._parent.toggleMenu();
			};
		}
		
		logger.debug("play " + playButton);
		logger.debug("pause " + pauseButton);
		logger.debug("fullscreen " + fullScreenButton);
		logger.debug("menu " + menuButton);
		logger.debug("tracker " + tracker);
		logger.debug("volume " + volume);
	}
	
	private function createChildren():Void {
		createBackground(getConfig().controlsOverVideo());
		// empty release handler to catch clicks
	}

	private function arrange():Void {
		logger.debug("arranging control bar with " + getSkin());
		getSkin().arrangeControlBar(this);
		originalWidth = _width;
	}

	public function showMenu():Boolean {
		return getConfig().showMenu();
	}

	public function showFullScreenButton():Boolean {
		return getConfig().showFullScreenButton();
	}
	
	public function getMaxControlHeight():Number {
		if (maxControlHeight == undefined)
			maxControlHeight = Util.maxHeight([playButton, pauseButton, nextButton, prevButton]);
		return maxControlHeight;
	}
	
	public function buttonsWidth():Number {
		var defaults:Number = (playAndPauseTogether() ? playButton._width : playButton._width + pauseButton._width)
			 + (showStopButton() ? stopButton._width : 0);
		if (! prevButton) return defaults;
		return defaults + prevButton._width + nextButton._width;
	}
	
	public function playAndPauseTogether():Boolean {
		return playButton._x == pauseButton._x;
	}

	public function setPlayVisible(visible:Boolean):Void {
		if (playAndPauseTogether()) {
			playButton._visible = visible;
			pauseButton._visible = ! visible;
		} else {
			playButton._visible = true;
			pauseButton._visible = true;
		}
	}

	public function showStopButton():Boolean {
		return getConfig().showStopButton();				
	}
	
	public function showVolumeSlider():Boolean {
		return getConfig().showVolumeSlider();
	}
	
	public function showMuteVolumeButton():Boolean {
		return getConfig().showMuteVolumeButton();
	}
	
	public function showPlayButton():Boolean {
		return getConfig().showPlayButton();
	}
	
	public function showPauseButton():Boolean {
		return getConfig().showPlayButton();
	}

	private function usePlayList():Boolean {
		var size:Number = getConfig().getPlayList().size();
		logger.debug("playlist size is " + size);
		return size > 1 && getConfig().showPlayListButtons();
		
	}
	
	public function play():Void {
		logger.debug("FLVDisplay.play()");
		if (control.isPaused()) {
			control.resume();
		} else {
			control.play(getConfig().getPlayList().currentClip());
		}
	}
	
	public function pause():Void {
		logger.debug("FLVDisplay.pause()");
		control.pause();
	};
	
	public function rewind():Void {
		logger.debug("FLVDisplay.rewind()");
		control.rewind();
		stopButton.enabled = false;		
		FlowButton(stopButton).switchFaces(false);
	}

	public function setEnabled(isEnabled:Boolean):Void {
		this.enabled = isEnabled;
		pauseButton.enabled = isEnabled;
		playButton.enabled = isEnabled;
		tracker.enabled = isEnabled;
		nextButton.enabled = isEnabled;
		prevButton.enabled = isEnabled;
		menuButton.enabled = isEnabled;
		
		// only enable stop button if not stopped
		if (isEnabled && (control.isPlaying() || control.isPaused()) && ! isOnStartSplash())
			stopButton.enabled = true;
		else
			stopButton.enabled = false;

		if (isEnabled)
			fader.brighten();
		else
			fader.dim();
	}
	
	private function isOnStartSplash():Boolean {
		return getConfig().getPlayList().currentIndex() == 0 && getConfig().getPlayList().firstClipIsSplash();
	}
	
	public function initStates() : Void {
		if (playAndPauseTogether()) {
			if (isLoaded(playButton))
				playButton._visible = ! getConfig().isAutoPlay() || control.isPaused();
			if (isLoaded(pauseButton))
				pauseButton._visible = ! playButton._visible;
		}
	}
	
	private function createBackground():Void {
		if (! getConfig().showControls()) return;
		if (! mcBackground)
			mcBackground = createEmptyMovieClip("mcBackground", getNextHighestDepth());
	}
	
	private function getSkin():Skin {
		if (mySkin) return mySkin;
		return super.getSkin();
	}
	
	public function setSkin(skin:Skin):Void {
		mySkin = skin;
		createBackground();
	}
	
	/**
	 * Callback for controller.
	 */
	public function onPause():Void {
		logger.debug("ControlBar::onPause()");
		setPlayVisible(true);
	}	

	/**
	 * Callback for controller.
	 */
	public function onPlay():Void {
		createTrackUpdateInterval();
		onResume();
	}
	
	/**
	 * Callback for controller.
	 */
	public function onStop():Void {
		NewAbstractMovieClip.logger.debug("FLVDisplay::onStop()");
		setPlayVisible(true);
		stopButton.enabled = false;
	}

	/**
	 * Callback for controller.
	 */
	public function onResume():Void {
		NewAbstractMovieClip.logger.debug("ControlBar::onResume()");
		setPlayVisible(false);
		this.setEnabled(getConfig().getPlayList().currentClip().isControlEnabled());
		if (getConfig().getPlayList().currentClip().isControlEnabled())
			stopButton.enabled = true;
	}	
	
	private function createTrackUpdateInterval():Void {
		if (trackIntervalId == -1) {
			updateProgress.control = control;
			updateProgress.tracker = tracker;
	
			trackIntervalId = setInterval(updateProgress, 200);
			logger.debug("FLVDisplay::created tracking update interval with id " + trackIntervalId);
		}
	}
	
	function updateProgress():Void {
//		if (arguments.callee.playList.currentClip().isImage() && ! arguments.callee.playList.currentClip().getDuration() > 0) return;
		var status:PlaybackStatus = arguments.callee.control.getPlaybackStatus();
		arguments.callee.tracker.update(status);
	}
	
	public function onToggled(button): Void {
		if (button instanceof MuteVolumeToggleButton) {
			ControlBar(arguments.callee.bar).muteVolumeToggled();
			return;
		}
		
		if (ControlBar(arguments.callee.bar).setFullScreenSettings(button)) {
			if (getConfig().useFlashNativeFullScreen()) {
				FLVDisplay(arguments.callee.display).fullScreenInit();
				if (getConfig().useHwScaling())
					Stage["fullScreenSourceRect"] = FLVDisplay(arguments.callee.display).getVideoRectangle();
				Stage["displayState"] = "fullScreen";
			} else {
				FlowPlayerExternalInterface.openFullScreen(getConfig());
				FLVDisplay(arguments.callee.display).fullScreenInit();
			}
		} else {
			if (getConfig().useFlashNativeFullScreen()) {
				Stage["displayState"] = "normal";
			} else {
				FlowPlayerExternalInterface.exitFullScreen();
			}
		}
	}
	
	private function onVolumeChanged(volume:Number):Void {
		if (! muteVolumeButton.isDownPressed() || volume == 0)
			dispatchVolumeEvent(volume);
	}
	
	public function muteVolumeToggled():Void {
		if (muteVolumeButton.isDownPressed()) {
			if (volume)
				volume.mute();
			else
				dispatchVolumeEvent(0);
		} else {
			if (volume)
				volume.unmute();
			else
				dispatchVolumeEvent(100);
		}
	}
	
	private function dispatchVolumeEvent(volume:Number):Void {
		dispatchEvent("onVolumeChanged", volume);
	}
	
	public function onSeek(tracker:ProgressTracker) {
		NewAbstractMovieClip.logger.debug("FLVDisplay::onSeek, tracker = " + tracker);
		control.seek(tracker.getSeekPosition());
	}
	
	public function setFullScreenSettings(button:AbstractToggleButton):Boolean {
		if (! button.isDownPressed()) {
			logger.debug("entering full screen");
			enableFullScreenSettings();
			return true;
		} else {
			disableFullScreenSettings();
			return false;
		}
	}
	
	private function enableFullScreenSettings():Void {
		var config:Config = getConfig();
		if (! config.useFlashNativeFullScreen())
			control.stopCurrent();
		config.setControlsWidth(config.getControlsWidth());
		config.setFullScreen(true);
		config.setEmbedWidth(config.getEmbedWidth());
		config.setEmbedHeight(config.getEmbedHeight());
		if (getConfig().useFlashNativeFullScreen()) {
			fullScreenButton.pressDown();
		}
	}
	
	public function disableFullScreenSettings():Void {
		logger.info("exiting full screen mode");
		getConfig().setFullScreen(false);
	}

	private function toggleMenu():Void {
		display.toggleMenu();
	}

	public function addProgressListener(listener:Object):Void {
		logger.debug("ControlBar; adding progress listener to " + tracker + ", listener " + listener);
		tracker.addProgressListener(listener);
	}

	private function addClipsToFader(clips:Array):Void {
		for (var i:Number = 0; i < clips.length; i++) {
			fader.addClip(clips[i]);
		}
	}
	
	public function lastButtonBeforeTracker():MovieClip {
		if (usePlayList()) return nextButton;
		if (playButton) return playButton;
		if (stopButton) return stopButton;
		return undefined;
	}
	
	public function maxWidgetHeight():Number {
		return maxHeight(childClips);
	}

	public function getOriginalWidth():Number {
		return originalWidth;
	}
	
	public function enableFullScreenButton():Void {
		if (fullScreenButton)
			fullScreenButton.enabled = true;
	}
}