﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.audio.MP3Controller;
import org.flowplayer.config.Config;
import org.flowplayer.config.ExtendedConfig;
import org.flowplayer.config.HierarchicalConfig;
import org.flowplayer.config.RTMPConfigLoader;
import org.flowplayer.FlowPlayerExternalInterface;
import org.flowplayer.FLVController;
import org.flowplayer.FLVDisplay;
import org.flowplayer.image.ImageControl;
import org.flowplayer.MediaControl;
import org.flowplayer.MixedMediaControl;
import org.flowplayer.playlist.Clip;
import org.flowplayer.SWFController;
import org.flowplayer.thumbs.SuggestionControl;

import cinqetdemi.JSONConfig;

/**
 * FlowPlayer is the main video playback component.
 */
class org.flowplayer.FlowPlayer extends NewAbstractMovieClip {
	private var externalConfig:JSONConfig;
	private static var config:ExtendedConfig;
	private var rtmpConfigLoader:RTMPConfigLoader;
	
	private var display:FLVDisplay;
	private var control:MediaControl;
	private var externalInterface:FlowPlayerExternalInterface;
	private var initialized:Boolean;

	public function FlowPlayer() {
		logger.debug("In FlowPlayer constructor");
		addConfig(createFlashVarsConfig());
		initCustomConfig();
		this.setConfig.player = this;
		FlowPlayerExternalInterface.addCallback("setConfig", this, this.setConfig);
	}
	
	private function createFlashVarsConfig():ExtendedConfig {		
		if(_level0.config)
			return new HierarchicalConfig(_level0.config, "flashVars configuration");
		else
			return new HierarchicalConfig({}, "config with defaults");
		return null;
	}
	
	private function addConfig(newConfig:ExtendedConfig):Void {
		if (! newConfig) return;
		
		if (config)
			config.addChild(newConfig);
		else
			config = newConfig;
	}
	
	private function initCustomConfig():Void {
//		config = new HierarchicalConfig(SapoConfigFactory.getConfigObject(), "sapo config");
//		addConfig(new HierarchicalConfig(RoccoConfigFactory.getConfigObject(), "rocco config"));
		
		// predefined external config -->
//		if (! config.isFullScreen()) {
//			var fileName:String = Util.addBaseURL(config.getBaseURL(), "foobar.js");
//			if (config)
//				config.setConfigFileName(fileName);
//			else
//	 			addConfig(new HierarchicalConfig("{ configFileName: '"+ fileName +"' }", "custom config"));
// 		}
		// <-- predefined external config
	}

	private function doSetConfig(configObj:Object, name:String):Void {
		config = new HierarchicalConfig(configObj, name);
	}
	
	function onLoad():Void  {		
		logger.debug("FlowPlayer::onLoad");
		
		var waitForConfigs:Boolean = loadConfigurations();
		if (waitForConfigs) { 
			FlowPlayerExternalInterface.call("onFlowPlayerReady");
			return;
		}
		initPlayer();
		FlowPlayerExternalInterface.call("onFlowPlayerReady");
	}

	private function loadConfigurations():Boolean {
		
		var waitForInjectedConfig:Boolean = useInjectedConfig();
		if (waitForInjectedConfig) return true;
	
		var waitForExternalConfig:Boolean = loadExternalConfig();
		if (waitForExternalConfig) return true;

		logger.debug("Will use flashVars only as configuration");
	}

	private function useInjectedConfig():Boolean {
		if (getConfig().useConfigInject()) {
			logger.debug("Waiting for JavaScript to set the config");
			return true;
		}
		return false;		
	}

	public function setConfig(configObj:Object):Void {
		logger.debug("setConfig(): received config");
		if (config.getProperty("locked")) {
			logger.error("config is locked");
			return;
		}
		
		config = new HierarchicalConfig(configObj, "injected configuration");
		getSkin().setConfig(config);

		var waitForExternalConfig:Boolean = loadExternalConfig();
		if (waitForExternalConfig) {
			logger.debug("waiting for external config");
			return;
		}
		logger.debug(arguments.callee.player);
		arguments.callee.player.initPlayer();
	}
	
	private function loadExternalConfig():Boolean {
		var configFile:String = config.getConfigFileName();
		if (configFile) {
			loadExternalConfigUsingHttp(configFile);
			return true;
		}
		
		var rtmpConfigUrl:String = config.getRtmpConfigUrl();
		if (rtmpConfigUrl) {
			loadExternalConfigUsingRtmp(rtmpConfigUrl);
			return true;
		}
		
		return false;
	}
	
	private function loadExternalConfigUsingHttp(configFile:String):Void {
		logger.debug("Will use config file " + configFile);
		externalConfig = new JSONConfig();
		externalConfig.addConfigLoadListener(this);
		externalConfig.addLoadErrorListener(this);
		externalConfig.fileName = configFile;
		externalConfig.loadConfig();
	}
		
	private function loadExternalConfigUsingRtmp(url:String):Void {
		logger.debug("Will request config file from " + url);
		rtmpConfigLoader = new RTMPConfigLoader();
		rtmpConfigLoader.addListener(RTMPConfigLoader.EVENT_CONFIG_LOADED, this);
		rtmpConfigLoader.addListener(RTMPConfigLoader.EVENT_CONFIG_LOAD_ERROR, this);
		MovieClip(this).onRtmpConfigLoadError = onConfigLoadError;
		rtmpConfigLoader.load(url, config.getRtmpConfigParam());
	}
	
	public function onConfigLoadError():Void {
		logger.error("Failed to load config, will fallback to defaults");
		initPlayer();
	}
	
	public function onConfigLoaded():Void {		
		NewAbstractMovieClip.logger.debug("Recieved config loaded event for " + externalConfig.getConfigObject());
		externalConfig.removeConfigLoadListener(this);
		externalConfig.removeConfigLoadErrorListener(this);
//		config = new HierarchicalConfig(externalConfig.getConfigObject(), "external file configuration");
		addConfig(new HierarchicalConfig(externalConfig.getConfigObject(), "external file configuration"));
		initPlayer();
	}
	
	private function onRtmpConfigLoaded(config:String):Void {
		addConfig(new HierarchicalConfig(config, "external RTMP configuration"));
		initPlayer();
	}	

	public function initPlayer():Void {
		logger.debug("init");
		if (initialized) {
			logger.debug("Player already initialized, recreating display, resetting and starting the playlist");
			control.reset();
			display.cleanup();
			display.removeMovieClip();
		}
		logger.debug("initializing");
		
		this._visible = false;
		logger.debug("starting stream");

		Stage.align = "LT";
		Stage.scaleMode = "noScale";
		Stage.addListener(this);
		
		attachMovie("FLVDisplay", "display", getNextHighestDepth());
		createMediaControllers();
		startConfiguredStreams();
		_visible = true;
		stop();
		
		if (initialized)
//			display.setSize(__width, __height, true);
			display.arrangeVideo();
		else
			onResize();
		initialized = true;
	}
	
	private function initConfig():Void {
	}
	
	public function internetExplorerResizeHack():Void {
		NewAbstractMovieClip.logger.debug("FlowPlayer::internetExplorerResizeHack()");
		if ((Stage.width > 0) && (Stage.height > 0)) {
			this.onResize();
		}
		delete this.onEnterFrame;
	}
	
	public function onResize():Void {
		NewAbstractMovieClip.logger.info("FlowPlayer:: onResize(), " + getDimensions(this));
        this.setSize(Stage.width, Stage.height);
	}
	
	private function createMediaControllers():Void {
		this.control = createControllerFor(getMediaTypes(), config.getNoVideoClip(), config.isLooping());
		this.externalInterface = new FlowPlayerExternalInterface(config, control);
//		this.externalInterface = new RoccoExternalInterface(config, control);
		FlowPlayerExternalInterface.allCallbacksRegistered();
		display.init(control, externalInterface);
	}

	private function startConfiguredStreams():Void {
		if (! config.getPlayList()) {
			logger.debug("initial config has no playLists, will not start playback");
			return;
		}
		
		if (config.getPlayList().firstClipIsSplash()) {
			logger.info("first clip is a splash");
			playWithSplash();
			return;
		}

		if (config.isAutoPlay()) {
			logger.info("FlowPlayer:: Starting playback");
			control.play(config.getPlayList().currentClip());	
		} else if (config.isAutoBuffering()) {
			logger.info("FlowPlayer:: Starting buffering");
			control.startBuffering(config.getPlayList().currentClip());
		}
	}
	
	private function playWithSplash():Void {
		if (config.isAutoPlay()) {
			logger.info("autoPlaying from fist non-spash clip");
			config.getPlayList().toIndex(1, true);
			control.play(config.getPlayList().currentClip());	
		} else {
			logger.info("autoPlay == false, will pause on splash");
			control.play(config.getPlayList().currentClip());
		}
	}
	
	private function createControllerFor(mediaTypes:Array, noVideoClip:Clip, loop:Boolean):MediaControl {
//		var mediaTypes:Array = playList.getClipTypes();

		var controllers:Array = new Array();
		for (var i : Number = 0; i < mediaTypes.length; i++) {
			var type:String = mediaTypes[i];
			controllers[type] = createControllerForType(type);
		}

		if (noVideoClip.getType() == 'jpg' && ! controllers['jpg'])
			controllers['jpg'] = createControllerForType('jpg');
		if (noVideoClip.getType() == 'png' && ! controllers['png'])
			controllers['png'] = createControllerForType('png');
		if (noVideoClip.getType() == 'mp3' && ! controllers['mp3'])
			controllers['mp3'] = createControllerForType('mp3');
		
		logger.info("following media types were initialized:");
		logger.info(controllers);
		
		return new MixedMediaControl(config.getPlayList(), noVideoClip, controllers, loop, config.isAutoBuffering(), config.isAutoRewind());
	}

	private function createControllerForType(type:String):MediaControl {
		logger.debug("creating controller for type " + type);
		if (type == 'flv' || type == 'video')
			return new FLVController(config);
		else if (type == 'jpg' || type == 'png' || type == 'overlay')
			return new ImageControl(config.getSkin());
		else if (type == 'swf')
			return new SWFController(config.getMainBufferLength());
		else if (type == 'suggestions')
			return new SuggestionControl(config.getPlayList());
		else if (type == 'mp3')
			return new MP3Controller();
	}
	
	private function arrange():Void {
		display.setSize(__width, __height);
		display._x = 0;
		display._y = 0;
	}
	
	public static function getConfig():Config {
		return config;
	}

	private function getMediaTypes() : Array {
		if (config.getPlayList()) return config.getPlayList().getClipTypes();
		return Clip.TYPES_ALL;
	}

}
