/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowplayer.config.Config;
import org.flowplayer.MediaControl;
import org.flowplayer.MediaSize;
import org.flowplayer.PlaybackStatus;
import org.flowplayer.playlist.Clip;
import org.flowplayer.playlist.PlayList;

import flash.external.ExternalInterface;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;
/**
 * @author anssi
 */
class org.flowplayer.FlowPlayerExternalInterface {

	private static var logger:Logger = new Logger();
	private var control:MediaControl;
	private static var playerId;
	private static var localConnection:LocalConnection;
	private static var config:Config;
	private var previousPlaybackTime : Number;
	
	public function FlowPlayerExternalInterface(config:Config, control:MediaControl) {
		logger.addPublisher(new ConsolePublisher());
		this.control = control;
		FlowPlayerExternalInterface.config = config;
		logger.debug("registerCallbacks, config :");
		logger.debug(config);
		registerCallbacks();
		if (! config.useFlashNativeFullScreen() && config.getFullScreenScriptURL())
			createFullScreenOpener(config.getFullScreenScriptURL());
	}
	
	private function registerCallbacks() : Void {
		if (! ExternalInterface.available)
			logger.info("ExternalInteface is not available in this runtime. JavaScript access will be disabled.");
		try {
			// media control
			addCallback("DoPlay", control, control.playCurrent);
			addCallback("DoStop", control, control.stop);
			addCallback("Reset", control, control.reset);
			addCallback("StartBuffering", control, control.startBuffering);
			addCallback("Pause", control, control.pause);
			addCallback("Seek", control, control.seekToTime);
			addCallback("ToggleLoop", control, control.toggleLoop);
			addCallback("getTime", this, this.getTime);
			addCallback("getDuration", this, this.getDuration);
			addCallback("getPercentLoaded", this, this.getPercentLoaded);
			addCallback("getIsPlaying", control, control.isPlaying);
			addCallback("getIsPaused", control, control.isPaused);
			addCallback("getIsLooping", control, control.isLooping);
			
			// new, v. 1.21
			addCallback("setPlayerId", this, FlowPlayerExternalInterface.setPlayerId);
			addCallback("getPlayerId", this, FlowPlayerExternalInterface.getPlayerId);
			addCallback("addCuePoint", control, control.addCuePoint);
			addCallback("addCuePoints", control, control.addCuePoints);
			addCallback("playClip", this, this.playClip);
			addCallback("setClip", this, this.setClip);

			// playlist control
			if (config.getPlayList()) {
				createPlayListCallbacks(config.getPlayList());
			}
		} catch (e:Error) {
			logger.error("Unable to add callback to ExternalInterface: " + e);
		}
	}
	
	private function createPlayListCallbacks(playList:PlayList):Void {
		addCallback("hasNext", playList, playList.hasNext);
		addCallback("NextClip", playList, playList.next);
		addCallback("PrevClip", playList, playList.prev);
		addCallback("getPlayListSize", playList, playList.size);
		addCallback("getCurrentClip", playList, playList.currentIndex);
		addCallback("ToClip", playList, playList.toIndex);
	}

	public static function addCallback(methodName:String, obj:Object, func:Function) : Void {
		logger.addPublisher(new ConsolePublisher());
		
		if (localConnection == null)
			localConnection = new LocalConnection();
		localConnection[methodName] = func;
		
		var success:Boolean = ExternalInterface.addCallback(methodName, obj, func);
		if (success)
			logger.info("Added callback: method = '" + methodName + "', obj = " + obj + ", func = " + func);
		else		
			logger.error("Unable to register callback for " + methodName + " on target object " + obj);
	}
	
	public static function allCallbacksRegistered():Void {
		localConnection.connect("FlowPlayer");
	}
		
	public function getTime():Number {
		var status:PlaybackStatus = control.getPlaybackStatus();
		return status.time;
	}
	
	public function getDuration():Number {
		var status:PlaybackStatus = control.getPlaybackStatus();
		logger.debug("ExternalInterface.getDuration: " + status);
		return status.totalDuration;
	}
	
	public function getPercentLoaded():Number {
		var status:PlaybackStatus = control.getPlaybackStatus();
		logger.debug("ExternalInterface.getPercentLoaded: " + status);
		if (config.getStreamingServerURL())
			return 100 * (status.time + status.currentBufferLength) / status.totalDuration;
		else
			return 100  * status.currentBufferLength / status.totalBufferSize;
	}

	public static function call(functionName:String, param:Object):Void {
		if (config.getGoogleAnalyticsPrefix())
			sendGoogleAnalytics(functionName, param);
//		if (PlumExternalInterface.call(functionName, param))
//		if (RoccoExternalInterface.call(functionName, param))
			ExternalInterface.call(functionName, param);
	}
	
	private function createFullScreenOpener(fullScreenScript:String):Void {
		var js:String = "javascript: { ";
		js += "var opener = document.getElementById('flowplayerfullscreenopener');";
		js += "if (! window.flowPlayerOpenFullScreen) {";
		js += "var script = document.createElement('script');";
		js += "script.id = 'flowPlayerOpenFullScreen';";
		js += "script.type = 'text/javascript';";
		js += "script.src = '"+ fullScreenScript +"';";
		js += "var head = document.getElementsByTagName('head').item(0);";
		js += "head.appendChild(script);";
		js += "}";
		js += "}";
		js += "void 0;";
		getURL(js);
	}
	
	public static function openFullScreen(config:Config):Void {
		prepareConfigForExternalPages(config);
		config.setIninitalMediaSize(MediaSize.FITTED_PRESERVING_ASPECT_RATIO);
		call("flowPlayerOpenFullScreen", config.stringify());
	}
	
	public static function getConfigStringForEmailLink(config:Config):String {
		prepareConfigForExternalPages(config);
		var currentSizeSetting:Number = config.getInitialMediaSize();
		config.setIninitalMediaSize(MediaSize.FITTED_PRESERVING_ASPECT_RATIO);
		var result:String = escape(config.stringify());
		config.setIninitalMediaSize(currentSizeSetting);
		return result;
	}

	private static function prepareConfigForExternalPages(config:Config):Void {
		if (config.getConfigFileName())
			config.setConfigFileName(undefined);
		if (config.getBaseURL() == undefined)
			config.setBaseURL(getBaseURLFromRootUrl());
	}
	
	public static function getConfigStringForEmbed(config:Config):String {
		config.setFullScreen(undefined);
		if (config.getBaseURL() == undefined)
			config.setBaseURL(FlowPlayerExternalInterface.getBaseURLFromRootUrl());
		config.setIsEmbedded(true);
		return config.stringify();
	}

	public static function exitFullScreen() : Void {
		call("flowPlayerExitFullScreen");		
	}
	
	public static function getBaseURLFromRootUrl():String {
		var url:String = _root._url;
		return url.substring(0, url.substring(0, url.indexOf(".swf")).lastIndexOf("/"));
	}
	
	public static function getPlayerSWFUrl():String {
		return _root._url.substring(0, _root._url.indexOf(".swf")+4);
	}
	
	public static function setPlayerId(playerId):Void {
		FlowPlayerExternalInterface.playerId = playerId;
		logger.debug("playerID was set to " + FlowPlayerExternalInterface.playerId);
	}
	
	public static function getPlayerId() {
		return FlowPlayerExternalInterface.playerId;
	}

	public function playClip(clipObj:Object):Void {
		var newPlayList:PlayList = setClip(clipObj);
		control.play(newPlayList.currentClip());
	}
	
	private function setClip(clipObj:Object):PlayList {
		var playList:PlayList = PlayList.fromObjectsWithProps(undefined, [ clipObj ]);
		config.setPlayList(playList);
		control.setPlayList(playList);
		createPlayListCallbacks(playList);
		control.stop();
		return playList;		
	}

	private static function sendGoogleAnalytics(functionName:String, param:Object):Void {
		if (! param instanceof Clip) return;
		var clip:Clip = Clip(param);
//		if (clip.isImage() && ! clip.getDuration()) return;
		var eventName:String = getAnalyticsEventName(functionName);
		logger.debug("seding google analytics for event " + eventName);
		logger.debug(config.getGoogleAnalyticsPrefix());
		if (eventName)
			getURL("javascript:urchinTracker('/" + config.getGoogleAnalyticsPrefix() + "/" + eventName + "/" + clip.getFilename() + "')");
	}

	private static function getAnalyticsEventName(functionName:String):String {
		if (functionName == "onStartBuffering")
			return "start";
		if (functionName == "onPlay")
			return "start";
		if (functionName == "onClipDone")
			return "finish";
		if (functionName == "onStop")
			return "stop";
		if (functionName == "onPause")
			return "pause";
		if (functionName == "onResume")
			return "resume";
		if (functionName == "onFullScreenInit")
			return "fullscreen";
		if (functionName == "onFullScreenExit")
			return "reduce";
		return null;
	}

	private function onPlaybackTimeProgress(timeObj:Object):Void {
		var time:Number = Number(timeObj);
		if (previousPlaybackTime != time) {
			call("onPlaybackTime", time);
		}
		this.previousPlaybackTime = time;	
	}
}