import org.flowlib.NewAbstractMovieClip;

/**
 * @author anssi
 */
class org.flowplayer.FlowText extends NewAbstractMovieClip {
	
	private var textField:TextField;
	private var colors:Array;
	private var alphas:Array;
	private var ratios:Array;
	private var borderGradient:Boolean;
	private var originalTextSize:Number;
	private var maxCharsForOrigSize:Number;
	
	public function FlowText() {
		super();
		colors = [0x888888, 0x888888];
		alphas = [100, 0];
		ratios = [128, 255]; 
		borderGradient = true;
	}
	
	public function set drawBorderGradient(newValue:Boolean):Void {
		logger.debug("setting border gradient to " + newValue);
		borderGradient = newValue;
	}
	
	public function set text(newText:String):Void {
		textField.text = newText;
		adjustTextSize();
	}
	
	private function adjustTextSize():Void {
		var format:TextFormat = textField.getNewTextFormat();
		if (textField.text.length <= maxCharsForOrigSize) {
			// restore original size
			format.size = originalTextSize;
			textField.setNewTextFormat(format);
			return;
		}

		if (format.size < originalTextSize)
			return;
			
		changeFontSize(-2);
	}
	
	public function get text():String {
		return textField.text;
	}
	
	public function setTextProperties(newAlign:String, color:Number, size:Number, maxCharsForThisSize:Number):Void {
		this.maxCharsForOrigSize = maxCharsForThisSize;
		setTextFormat(newAlign, color, size || 10);
	}

	private function createChildren():Void {
		drawBorder();
		createField();
	}
	
	private function arrange():Void {
		logger.debug("FlowText.arrange: " + getDimensions(this));
//		this._width = __width;
		textField._x = 0;
		textField._width = __width;
//		textField._height = __height;
		drawBorder();
	}
	
	public function drawBorder():Void {
		if (this.__width == undefined) {
			this.__width = 80;
			this.__height = 14;
		}
		if (! borderGradient) return;
		
		this.clear();
		var width:Number = __width;
		logger.error("drawBorder: " + __width + ", " + __height);
		var matrix = { matrixType:"box", x:0, y:0, w:width, h:__height, r:Math.PI * 2};
		
		beginGradientFill("linear", colors, alphas, ratios, matrix);
		drawRectangle(this, 0, 0, width, __height, false);
		endFill();
	}
	
	private function createField() {
		logger.debug("createField: " + __width + ", " + __height);
		createTextField("textField", getNextHighestDepth(), 0, 0, __width, 20);
		logger.debug(getDimensions(this));
//		textField.embedFonts = true;
		setTextFormat("left", 0x000000, 15);		
	}
	
	private function changeFontSize(sizeChange:Number) : Void {
		var format:TextFormat = textField.getNewTextFormat();
		format.size = format.size + sizeChange;
		textField.setNewTextFormat(format);
		logger.debug("degreased font size by one");
	}
	
	private function setTextFormat(textAlign:String, color:Number, size:Number):Void {
		originalTextSize = size;
		var format:TextFormat = getDefaultTextFormat();
		if (size)
			format.size = size;
		format.align = textAlign;
		format.color = color;
		format.leftMargin = 5;
		format.bold = true;
		format.font = "_typewriter";
		textField.setNewTextFormat(format);
		logger.debug(this._name + ", text set to " + textField.text);
	}
	
	public function set borderColors(colors:Array):Void {
		this.colors = colors;
	}
	
	public function set borderRatios(ratios:Array):Void {
		this.ratios = ratios; 
	}
	
	public function set borderAlphas(alphas:Array):Void {
		this.alphas = alphas;
	}

}