/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowplayer.MediaSize;
import org.flowlib.NewAbstractMovieClip;
import LuminicBox.Log.*; 

class org.flowplayer.MediaResizer {
	
	private var media:MovieClip;
	private var origWidth:Number;
	private var origHeight:Number;
	private var maxWidth:Number;
	private var maxHeight:Number;
	private var logger:Logger;
	private var currentSizingOption:Number;
	
	public function MediaResizer(media:MovieClip, maxWidth:Number, maxHeight:Number) {
		this.media = media;
		this.origWidth = media._width;
		this.origHeight = media._height;
		this.maxWidth = maxWidth;
		this.maxHeight = maxHeight;
		this.logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
		currentSizingOption = MediaSize.FITTED_PRESERVING_ASPECT_RATIO;
	}
	
	public function setOrigSize(widht:Number, height:Number) {
		if (! widht) return;
		if (! height) return;
		this.origWidth = widht;
		this.origHeight = height;
		logger.debug("MediaResizer.setOrigSize: origWidth " + origWidth + ", origHeight " + origHeight + ", maxWidth " + maxWidth + ", maxHeight " + maxHeight);
	}
	
	public function setMaxSize(maxWidth:Number, maxHeight:Number) {
		this.maxWidth = maxWidth;
		this.maxHeight = maxHeight;		
//		resizeToCurrentOption();
	}
		
	public function resizeTo(sizingOption:Number, force:Boolean):Void {
		logger.debug("starting resize with option " + sizingOption);
		logger.debug("origWidth " + origWidth + ", origHeight " + origHeight + ", maxWidth " + maxWidth + ", maxHeight " + maxHeight);
		if (! sizingOption)
			sizingOption = currentSizingOption;
		
		if (sizingOption == MediaSize.FITTED_PRESERVING_ASPECT_RATIO || sizingOption == MediaSize.FULL_SCREEN) {
			resizeToFit();
		} else if (sizingOption == MediaSize.HALF_FROM_ORIGINAL) {
			resizeToHalfAvailableSize();
		} else if (sizingOption == MediaSize.ORIGINAL) {
			resizeToOrig(force);
		} else if (sizingOption == MediaSize.FILLED_TO_AVAILABLE_SPACE) {
			resizeToMax();
		}
		logger.debug("resized to option " + sizingOption);
		this.currentSizingOption = sizingOption;
	}
	
	private function resizeToCurrentOption():Void {
		return;
		resizeTo(currentSizingOption);
	}
		
	private function resizeToFit() : Void {
		logger.debug("MediaResizer.resizeToFit: " + media._name + ", origWidth " + origWidth + ", origHeight " + origHeight + ", maxWidth " + maxWidth + ", maxHeight " + maxHeight);
		var xRatio:Number = maxWidth / origWidth;
		
		if (xRatio * origHeight <= maxHeight) {
			scale(xRatio);
		} else {
			scale(maxHeight / origHeight);
		}
	}
		
	public function scale(scalingFactor:Number):Void {
		resize(scalingFactor * origWidth, scalingFactor * origHeight);
	}

	private function resizeToOrig(force:Boolean) : Void {
		if (force) {
			resize(origWidth, origHeight);
			return;
		}
		if (origHeight > maxHeight || origWidth > maxWidth) {
			logger.debug("calling resizeToFit()");
			resizeToFit();
		} else if (origWidth && origHeight) {
			resize(origWidth, origHeight);
		} else {
			logger.debug("calling resizeToFit()");
			resizeToFit();
		}
	}

	private function resizeToHalfAvailableSize() : Void {
		scale((maxWidth / 2) / origWidth);
	}
	
		
	private function resizeToMax():Void {
		resize(maxWidth, maxHeight);
	}

	private function resize(newWidth:Number, newHeight:Number):Void {
		if (media instanceof NewAbstractMovieClip) {
			NewAbstractMovieClip(media).setSize(newWidth, newHeight, true);
		} else {
			media._width = newWidth;
			media._height = newHeight;			
		}
	}
			//

	public function get currentSize():Number {
		return currentSizingOption;
	}
	
	public function resetPosition():Void {
		media._x = 0;
		media._y = 0;
	}
	
	public function get originalWidth():Number {
		return origWidth;
	}

	public function get originalHeight():Number {
		return origHeight;
	}
	
	public function hasOrigSize():Boolean {
		return origHeight != undefined && origWidth != undefined;
	}
}
