﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.AlphaCycleEffect;
import org.flowlib.Observable;
import org.flowplayer.FlowPlayerExternalInterface;
import org.flowplayer.MediaContext;
import org.flowplayer.MediaControl;
import org.flowplayer.MediaResizer;
import org.flowplayer.MediaView;
import org.flowplayer.PlaybackStatus;
import org.flowplayer.Util;
import org.flowplayer.playlist.Clip;
import org.flowplayer.playlist.PlayList;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;

class org.flowplayer.MixedMediaControl extends Observable implements MediaControl {
	
	private var controllers:Array;
	private var displays:Array;
	private var resizers:Array;
	private var looping:Boolean;
	private var playList:PlayList;
	private var noMediaFoundClip:Clip;
	public var logger:Logger;
	private var paused:Boolean = false;
	private var playing:Boolean = false;
	private var rewinding:Boolean = false;
	private var prevIndex:Number;
	private var context:MediaContext;
	private var autoBuffer : Boolean;
	private var autoRewind : Boolean;
	private var sound:Sound;

	public function MixedMediaControl(playList:PlayList, noMediaFoundClip:Clip, controllersByMedia:Array, loop:Boolean, autoBuffer:Boolean, autoRewind:Boolean) {
		super();
		this.logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
		this.controllers = controllersByMedia;
		this.displays = new Array();
		this.noMediaFoundClip = noMediaFoundClip;
		this.looping = loop;
		this.autoBuffer = autoBuffer;
		this.autoRewind = autoRewind;
		if (playList)
			setPlayList(playList);

		for (var key in controllersByMedia) {
			controllers[key].addListener("onLoadBegin", this);
			controllers[key].addListener("onStartBuffering", this);
			controllers[key].addListener("onBufferFull", this);
			controllers[key].addListener("onBufferFlush", this);
			controllers[key].addListener("onMetaData", this);
			controllers[key].addListener("onClipDone", this);
			controllers[key].addListener("onSelfPause", this);
			controllers[key].addListener("onStreamNotFound", this);
			controllers[key].addListener("onPlayListChangedStartPlay", this);
			controllers[key].addListener("onPlayInitiated", this);
		}
	}
	
	public function setContext(context:MediaContext):Void {
		this.context = context;
		for (var i:Number = 0; i < controllers.length; i++) {
			MediaControl(controllers[i]).setContext(context);
		}
	}
	
	public function attachView(mcDisplay:MovieClip, mediaType:String):Void {
		displays[mediaType] = mcDisplay;
		controllers[mediaType].attachView(mcDisplay);
		mcDisplay._visible = false;
		mcDisplay.controller = this;
		
		if (! handlesOnRelease(mcDisplay)) {
			mcDisplay.onRelease = function():Void {
				if (! this.firstClick || getTimer() - this.firstClick > 500) {
					this.controller.displayClicked();
				}
				this.firstClick = getTimer();
			};
		}
		mcDisplay.enabled = true;
	}
	
//
//	private function setDisplayClickEnabled():Void {
//		if (handlesOnRelease(getDisplay()))
//			getDisplay().enabled = true;
//		else
//			getDisplay().enabled = playList.currentClip().getLinkURL() != undefined;
//	}
	
	private function handlesOnRelease(clip:MovieClip):Boolean {
		if (! clip instanceof MediaView) return false;
		return MediaView(clip).handlesOnRelease();
	}
	
	public function attachVideo(video:Video, mediaType:String):Void {
		controllers[mediaType].attachVideo(video);
	}

	public function toggleLoop() : Void {
		this.looping = ! this.looping;
	}

	public function isLooping() : Boolean {
		return this.looping;
	}

	public function startBuffering(clip:Clip) : Void {
//		dispatchEvent("onStartBuffering");
		logger.info("MixedMediaController::startBuffering");
		getController().startBuffering(clip);
	}

	public function playCurrent() : Void {
		logger.info("MixedMediaController::playCurrent");
		play(playList.currentClip());
	}

	public function play(clip:Clip) : Void {
		if (playing && ! paused) return;
		dispatchEvent("onPlay");
		logger.info("MixedMediaController::play");
		playing = true;
		
		if (paused) {
			paused = false;
			resume();
		} else {
//			switchDisplayVisibility();
			getController().play(clip);
		}
		FlowPlayerExternalInterface.call("onPlay", clip);
	}

	public function stop() : Void {
		goToStartOfPlayList();
	}

	public function stopCurrent():Void {
		dispatchEvent("onStop");
		getController().stop();
		playing = false;
		paused = false;
		FlowPlayerExternalInterface.call("onStop", playList.currentClip());
	}
	
	private function stopToIndex(index:Number):Void {
		dispatchEvent("onStop");
		logger.info("MixedMediaController::stopToIndex " + index);
		playing = false;
		paused = false;
		
		playList.toIndex(index);
		for (var key in controllers) {
			controllers[key].stop();
		}
		FlowPlayerExternalInterface.call("onStop", playList.currentClip());
	}
	
	public function reset():Void {
		logger.info("MixedMediaController::reset");

		playList.toIndex(0);
		for (var key in controllers) {
			controllers[key].reset();
		}		
	}

	public function pause() : Void {
		dispatchEvent("onPause");
		logger.info("MixedMediaController::pause");
		paused = true;
		getController().pause();
		FlowPlayerExternalInterface.call("onPause", playList.currentClip());
	}

	public function resume() : Void {
		logger.info("MixedMediaController::resume");
		paused = false;
		getController().resume();
		FlowPlayerExternalInterface.call("onResume", playList.currentClip());
		dispatchEvent("onResume");
	}
	
	public function displayClicked() {
		logger.debug("MixedMediaControl::displayClicked(), playing = " + isPlaying());
		
		var link:String = this.playList.currentClip().getLinkURL();
		if (link == '') return;
		if (link) {
			getURL(link, this.playList.currentClip().getLinkWindow());
		} else {
			if (isPlaying())
				pauseOrResume();
			else
				play(playList.currentClip());
		}
	}

	public function seek(percentage : Number) : Void {
		dispatchEvent("onSeek");
		getController().seek(percentage);
	}

	public function seekToTime(offset:Number) : Void {
		dispatchEvent("onSeek");
		if (paused && preBufferNext()) {
			var nextType:String = playList.peekNext().getType();
//			showDisplay(displays[nextType]);
			controllers[nextType].seekToTime(offset);
		} else {
			getController().seekToTime(offset);
		}
	}

	private function pauseOrResume() : Void {
		dispatchEvent("onPauseOrResume");
		paused = !paused;
		if (paused) {
			getController().pause();
		} else {	
			getController().resume();
		}
			
		var event:String = paused ? "onPause" : "onResume";
		FlowPlayerExternalInterface.call(event, playList.currentClip());
		dispatchEvent(event);
	}

	public function isPaused() : Boolean {
		return paused;
//		return getController().isPaused();
	}

	public function isPlaying() : Boolean {
		logger.debug("MixedMediaControl.playing " + playing + ", paused " + paused);
		return playing;
//		return getController().isPlaying();
	}

	public function getPlaybackStatus() : PlaybackStatus {
		if ((currentClipIsImage() && ! playList.currentClip().getDuration() > 0) || playList.currentClip().isOverlay()) {
			return controllers[playList.peekNext().getType()].getPlaybackStatus();
		}
		
		return getController().getPlaybackStatus();
	}

	public function addListener(eventName:String, targetObject:Object) {
		if (eventName == 'onLoadBegin' || eventName == 'onBufferFull' || eventName == 'onStartBuffering' || eventName == 'onBufferFlush' || eventName == 'onMetaData' || eventName == 'onCuePointWithThumb' || eventName == 'onClipDone')
			addListenerToChildControllers(eventName, targetObject);
		else
			super.addListener(eventName, targetObject);
	}

	private function addListenerToChildControllers(eventName : String, targetObject : Object) : Void {
		for (var key in controllers) {
			controllers[key].addListener(eventName, targetObject);
		}
	}

	public function onClipDone():Void {
		FlowPlayerExternalInterface.call("onClipDone", playList.currentClip());
		playList.currentClip().played();
		
		// TODO: show suggestions
//		if (playList.currentClip().getSuggestionsInfoUrl()) {
//			
//		}
		
		if (! playing) return;
		
		if (! playList.hasNext()) {
			if(looping){
				if (playList.size() == 1) {
					logger.debug("MixedMediaControl.onClipDone(): seeking to zero on current clip");
					getController().seek(0);
				} else if (playList.firstClipIsSplash() && playList.size() == 2) {
					logger.debug("MixedMediaControl.onClipDone(): seeking to zero in the only non-splash clip in the playlist (first clip is a splash)");
					getController().seek(0);
				} else if (playList.firstClipIsSplash()) {
					logger.debug("MixedMediaControl.onClipDone(): moving to clip 1 (first clip is a splash)");
					playList.toIndex(1);
				} else {
					logger.debug("MixedMediaControl.onClipDone(): moving to clip 0 (splash not in use)");
					playList.toIndex(0);
				}
			} else {
				logger.info("loop is off");
				if (autoRewind)
					rewind();
			}
		} else {
			playList.next();
		}
	}
	
	private function goToStartOfPlayList():Void {
		if (autoRewind || rewinding) {
			playList.toIndex(0);
			getController().seek(0);

			// we can stop because we are not looping
			if (playList.firstClipIsSplash()) {
				// the splash pauses
				paused = true;
			} else {
				logger.debug("first clip is not a splash, stopping");
				getController().stop();
			}
		} else {
			dispatchEvent("onStop");
			// move to the beginning silently, will not affect the tracker
			playList.toIndex(playList.firstClipIsSplash() ? 1 : 0, true);
		}
	}
	
	public function onStreamNotFound(clip:Clip):Void {
		FlowPlayerExternalInterface.call("onStreamNotFound", playList.currentClip());

		if (noMediaFoundClip) {
			controllers[clip.getType()].reset();
			displays[clip.getType()]._visible = false;
						
			var nextClipBeforeReplace:Clip = playList.peekNext();
			playList.replace(clip, noMediaFoundClip);
			
			logger.info("playing noMediaFoundClip " + noMediaFoundClip.getURL());
			playing = false;
			playCurrent();
		}
	}

	public function onClipChanged() : Void {
		FlowPlayerExternalInterface.call("onClipChanged", playList.currentClip());
		playList.currentClip().played();
		
		hidePreviousDisplay();
		if (! playList.currentClip().isShowOnLoadBegin())
			hideCurrentDisplay(true);

		logger.debug("MixedMediaControl.onClipChanged(): changed to clip " + playList.currentClip());
//		setDisplayClickEnabled();
		if (playing && ! paused) {
			logger.debug("MixedMediaControl.onClipChanged(): calling play on control " + getController());
			getController().play(playList.currentClip());
		} else {
			logger.debug("MixedMediaControl.onClipChanged(): calling stop/startBuffering on control " + getController());
			getController().stop();
			getController().startBuffering(playList.currentClip());
		}
	}
	
	public function onPlayListChangedStartPlay(playList:PlayList):Void {
		setPlayList(playList);
		play(playList.currentClip());
	}
	
	private function hideAllDisplays():Void {
		for (var type:String in displays) {
			var disp:MovieClip = MovieClip(displays[type]);
			if (disp._visible)
				disp._visible = false;
		}
	}
	
	public function onPlayInitiated():Void {
		stopToIndex(playList.currentIndex());
		play(playList.currentClip());
	}
	
	public function onLoadBegin(clip:Clip):Void {
		FlowPlayerExternalInterface.call("onLoadBegin", clip);
		
		// return if prebuffering next. 
		if (clip != playList.currentClip()) {
			var currentType = playList.currentClip().getType(); 	
			if (currentType == 'png' || currentType == 'overlay')
				AlphaCycleEffect.fade(displays[clip.getType()], true);
			return;
		}
		
		if (getController().allowCenterViewOnLoadBegin()) {
			logger.debug("centering " + getDisplay());
			Util.centerClip(getDisplay(), context.getDisplayWidth(), context.getDisplayHeight());
		}
		
		hidePreviousDisplay();
		if (clip.isShowOnLoadBegin()) {
			showCurrentDisplay();
		}
		prebufferFLV();
	}
	
	private function getPreviousType():String {
		return playList.allParents()[prevIndex].getType();
	}
	
	public function onStartBuffering(clip:Clip):Void {
		FlowPlayerExternalInterface.call("onStartBuffering", clip);
		if (getPreviousType() != 'overlay')
			hidePreviousDisplay();
	}
	
	public function onBufferFull(clip:Clip):Void {
		// following causes a browser crash if uncommented!
		//FlowPlayerExternalInterface.call("onBufferFull", clip);
		if (! clip.isShowOnLoadBegin())
			showCurrentDisplay();
	}
	
	public function onBufferFlush(clip:Clip):Void {
		FlowPlayerExternalInterface.call("onBufferFlush", clip);
	}
	
	public function onMetaData(obj:Object):Void {
		FlowPlayerExternalInterface.call("onMetaData", { duration: obj.duration, videodatarate: obj.videodatarate, audiodatarate: obj.audiodatarate, creationdate: obj.creationdate });
	}
	
	private function prebufferFLV():Void {
		if (preBufferNext()) {
			controllers[playList.peekNext().getType()].startBuffering(playList.peekNext());
		}
	}
	
	private function preBufferNext():Boolean {
		if (! autoBuffer) {
			logger.info("Autobuffering disabled, will not prebuffer FLV");
			return false;
		}
		var nextClip:Clip = playList.peekNext();
		if (nextClip.getType() == 'suggestions') return true;
		return currentClipIsImage() && nextClip.getType() == 'flv';
	}
	
	private function currentClipIsImage():Boolean {
		return playList.currentClip().isImage();
	}
		
	public function onSelfPause():Void {
		// if on last clip and not rewinding or looping --> must stop on last frame and after that continue from start
		if (! playList.hasNext() && ! isLooping() && ! autoRewind) {
			logger.debug("loop is false, autoRewind is false --> moving silently to the beginning of playList");
			playing = false;
			paused = false;
			playList.toIndex(playList.firstClipIsSplash() ? 1 : 0, true);
			return;
		}
		// for some reason the child controller paused himself, pause this also
		onPause();
		dispatchEvent("onPause");
		paused = true;
	}	
		
	public function onPause():Void {
		FlowPlayerExternalInterface.call("onPause", playList.currentClip());
	}	
//	
//	private function switchDisplayVisibility():Void {
//		hidePreviousDisplay();
//		showCurrentDisplay();
//	}
	
	private function hidePreviousDisplay(immediate:Boolean):Void {
		var index:Number = playList.currentIndex();
		var clips:Array = playList.allParents();
		var typeNow:String = clips[index].getType();
		var prevType:String = clips[prevIndex].getType();
		this.prevIndex = playList.currentIndex();
		
		logger.debug("typeNow " + typeNow);
		logger.debug("prevType " + prevType);

		if (typeNow == prevType)
			return;
		
		if (rewinding)
			controllers[prevType].rewind();
		else
			controllers[prevType].stop();
		rewinding = false;

		var prevDisp:MovieClip = displays[prevType];
		var dispNow:MovieClip = displays[typeNow];
		if (prevDisp == dispNow)
			return;
			
		if (hidesPreviousDisplay(typeNow)) {
			hideDisplay(displays[prevType], immediate);
		}
	}

	private function hidesPreviousDisplay(type:String):Boolean {
		return type != 'png' && type != 'overlay' && type != 'suggestions';
	}
	
	private function hideCurrentDisplay(immediate:Boolean):Void {
		hideDisplay(getDisplay(), immediate);
	}

	private function showCurrentDisplay():Void {
		var index:Number = playList.currentIndex();
		var clips:Array = playList.allParents();
		var typeNow:String = clips[index].getType();
		
		// TODO: Move to type specific displays
		if (getController().setViewVisibility()) {
			return;
		}
		
		context.resizeCurrentDisplay();
		logger.debug("showing display for type " + typeNow);
		logger.debug("display is " + displays[typeNow]);
		showDisplay(displays[typeNow]);
	}
	
	private function hideDisplay(disp:MovieClip, immediate:Boolean):Void {
		logger.error("hiding display " + disp + ", immediate " + immediate);
		if (immediate)
			disp._visible = false;
		else
			AlphaCycleEffect.fade(disp, false);
	}
	
	private function showDisplay(disp:MovieClip):Void {
		AlphaCycleEffect.fade(disp, true);
	}

	private function getController():MediaControl {
		return controllers[getType()];
	}
	
	private function getDisplay():MovieClip {
		return displays[getType()];
	}
	
	private function getType():String {
		return playList.currentClip().getType();
	}

	public function onVolumeChanged(volumePercentage:Number):Void {
		var newVolume:Number = volumePercentage * 2;
		sound.setVolume(newVolume);
		controllers["flv"].onVolumeChanged(newVolume);
//		controllers["mp3"].setVolume(volumePercentage);
	}

	public function addCuePoint(cueTimeSeconds : Number, cueName : String, cueObject : Object) : Void {
		controllers["flv"].addCuePoint(cueTimeSeconds, cueName, cueObject);
	}

	public function addCuePoints(cuePoints : Array) : Void {
		controllers["flv"].addCuePoints(cuePoints);
	}

	public function setPlayList(playList:PlayList):Void {
		this.playList = playList;
		this.playList.addListener(PlayList.CLIP_CHANGED_EVENT, this);
		this.prevIndex = playList.currentIndex();
		playing = false;
		paused = false;
		hideAllDisplays();
		dispatchEventWithObj("onPlayListChanged", playList);
	}

	public function setSound(sound:Sound):Void {
		this.sound = sound;
		for (var key:String in controllers) {
			MediaControl(controllers[key]).setSound(sound);
		}		
	}

	public function setViewVisibility():Boolean {
		return false;
	}

	public function allowCenterViewOnLoadBegin() : Boolean {
		return false;
	}
	
	public function rewind():Void {
		rewinding = true;
		goToStartOfPlayList();
	}
}