﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.flowplayer.ClipDescriptor;
import org.flowplayer.FlowText;
import org.flowlib.AbstractSlider;
import org.flowplayer.PlaybackStatus;
import org.flowplayer.Util;

/**
 * A scrubber that implements ProgressTracking.
 */
class org.flowplayer.ProgressTracker extends AbstractSlider {
	private static var EVENT_PLAYBACK_TIME_PROGRESS:String = "onPlaybackTimeProgress"; 
	private var previousDispatchTime: Number;
	private var color1:Number;
	private var color2:Number;

	private var oldLength : Object;

	public function ProgressTracker() {
		this.color1 = getSkin().getTimePlayedFontColor();
		this.color2 = getSkin().getTimeDisplayFontColor();
	}
	
	private function getChildDescriptors():Array {
		return [
			new ClipDescriptor("Dragger", "mcDragger", null)
		];
	}

	public function update(status:PlaybackStatus):Void {
		updatePlaybackStatus(status);
		updateBufferStatus(status);
	}

	public function addProgressListener(listener:Object):Void {
		addEventListener(EVENT_PLAYBACK_TIME_PROGRESS, listener);
	}

	private function updatePlaybackStatus(status:PlaybackStatus):Void {
		if (! status) {
			status = new PlaybackStatus(0, 0, 0, 0);
		}

		var progressRatio:Number = status.time / status.totalDuration; 
		setValue(progressRatio);
		setLabelText(Util.formatSeconds(status.time) + " / " + Util.formatSeconds(status.totalDuration));
		dispatchProgressEvent(status.time);
	}
	
	private function setLabelText(text:String) : Void {
		var field:TextField = mcLabel.text;
		field.text = text;
		field._x = 5;
		field._y = mcLabel._height / 2 - field._height / 2  - 1;
		colorLabel();
		setLabelLength(text, field.textWidth + 20);
	}
	
	private function setLabelLength(text:String, newLength:Number):Void {
		if (this.oldLength == text.length) return;
		mcLabel._width = newLength;
		getSkin().arrangeProgressTracker(this);
		this.oldLength = text.length;
	}
	
	public function colorLabel():Void {
		var field : TextField = mcLabel.text;
		var format:TextFormat = new TextFormat();
		format.color = color1;
		var pos:Number = field.text.indexOf("/", 0);
		field.setTextFormat(0, pos, format);
		format.color = color2;
		field.setTextFormat(pos + 1, field.text.length, format);
	}

	private function dispatchProgressEvent(time:Number):Void {
//		if (time - previousDispatchTime >= 1) {
			dispatchEvent(EVENT_PLAYBACK_TIME_PROGRESS, time);
			previousDispatchTime = time;
//		}
	}
	
	private function updateBufferStatus(status:PlaybackStatus):Void {
		var bufferRatio:Number = 0;
		if (arguments.length > 0) {
			bufferRatio = status.currentBufferLength / status.totalBufferSize;
				
			if (bufferRatio > 1)
				bufferRatio = 1;
		}
		setValue(bufferRatio, true);
	}

	private function arrangeSlider(mcSlider:AbstractSlider):Void {
		getConfig().getSkin().arrangeProgressTracker(mcSlider);
	}
		
	private function get barHeight():Number {
		return getConfig().getSkin().getProgressBarHeight();
	}
	
	private function get barXPos():Number {
		var valueFromSkin:Number = getConfig().getSkin().getProgressBarXPos();
		if (valueFromSkin >= 0) return valueFromSkin;
		return 0;
	}
	
	private function get barToRightEdgeGap():Number {
		return getConfig().getSkin().getProgressBarToRightEdgeGap();
	}
	
	private function onCreateBorder(mcBorder:MovieClip):Void {
		getConfig().getSkin().drawProgressBarBorderTo(mcBorder);
	}
	
	private function onCreateBackground(mcBack:MovieClip):Void {
		getConfig().getSkin().drawProgressBarBackgroundTo(mcBack);
	}

	private function allowSeekingBeyondValue2():Boolean {
		return getConfig().getStreamingServerURL() != undefined || getConfig().getStreamingServer() != undefined;
	}

	private function onCreateValueIndicator(mcValue:MovieClip):Void {
		getConfig().getSkin().onCreateProgressTrackerValueMc(mcValue);
	}

	private function onCreateValueIndicator2(mcValue:MovieClip):Void {
		getConfig().getSkin().onCreateProgressTrackerValue2Mc(mcValue);
	}

}