﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.skin.Skin;

/**
 * @author anssi
 */
class org.flowplayer.Util {
	
	private static var logger:Logger;
	
	private static  function logDebug(msg:String):Void {
		getLogger().debug(msg);
	}
	
	private static function getLogger():Logger {
		if (! logger) {
			logger = new Logger();
			logger.addPublisher(new ConsolePublisher());
		}
		return logger;
	}
	
	public static function addBaseURL(baseURL:String, fileName:String):String {
		if (fileName == undefined) return undefined;
		
		if (isCompleteURLWithProtocol(fileName)) return fileName;
		
		if (baseURL == '' || baseURL == null || baseURL == 'null') {
			return fileName;
		}
		if (baseURL != undefined) {
			return baseURL + "/" + fileName;
		}
		if (_root._url != undefined) {
			return _root._url.substring(0, _root._url.indexOf("FlowPlayer")) + fileName;
		}
		return fileName;
	}
	
	public static function isCompleteURLWithProtocol(fileName:String):Boolean {
		return fileName.indexOf("://") > 0;
	}
	
	static private function two(x:String) {
		return ((x>9) ? "" : "0") + x;
	}
	
	public static function formatSeconds(sec):String {
		var t:String = "";
		
		var min = Math.floor(sec/60);
		sec = int(sec) % 60;
		t = two(sec);
				
		var hr = Math.floor(min/60);
		min = min % 60;
		t = two(min) + ":" + t;
		
		if (hr == 0) return t;
		
		var day = Math.floor(hr/60);
		hr = hr % 60;
		t = two(hr) + ":" + t;
		
		if (day == 0) return t;
		
		t = day + ":" + t;
		
		return t;
	}
	
	public static function hasText(value:String):Boolean {
		if (value == undefined) return false;
		if (value == null) return false;
		return value.length > 0;
	}
				
//	public static function centerClip(clip:MovieClip, areaWidth:Number, areaHeight:Number):Void {
//		if (areaWidth)
//			clip._x = (areaWidth / 2) - (clip._width / 2);
//		if (areaHeight)
//			clip._y = (areaHeight / 2) - (clip._height / 2);
//	}
				
	public static function centerClip(clip:Object, areaWidth:Number, areaHeight:Number):Void {
		logger.debug("centerClip, is NewAbstractMovieClip == " + (clip instanceof NewAbstractMovieClip));
		var clipWidth:Number = clip instanceof NewAbstractMovieClip && 
			NewAbstractMovieClip(clip)._width > NewAbstractMovieClip(clip).width ? NewAbstractMovieClip(clip).width : clip._width; 
		var clipHeight:Number = clip instanceof NewAbstractMovieClip && 
			NewAbstractMovieClip(clip)._height > NewAbstractMovieClip(clip).height? NewAbstractMovieClip(clip).height : clip._height; 
		if (areaWidth)
			clip._x = (areaWidth / 2) - (clipWidth / 2);
		if (areaHeight)
			clip._y = (areaHeight / 2) - (clipHeight / 2);
	}
	
	public static function drawRectangleWithBorder(clip:MovieClip, x:Number, y:Number, width:Number, height:Number, lineThickness:Number, lineRGB:Number):Void {
//		logger.debug("drawing rectangle " + width + " x " + height + " on " + clip._name);
		if (lineThickness != null)
			clip.lineStyle(lineThickness, lineRGB, 100);
		//else
			//clip.lineStyle();
		drawRectangleNoFormatting(clip, x, y, width, height);
	}
	
	public static function drawRectangleNoFormatting(clip:MovieClip, x:Number, y:Number, width:Number, height:Number):Void {
		clip.moveTo(x + 1,  y);
		clip.lineTo(width, y);
		clip.lineTo(width, height);
		clip.lineTo(x, height);
		clip.lineTo(x, y);
	}
		
	public static function drawRectangle(clip:MovieClip, x:Number, y:Number, width:Number, height:Number, drawBorder:Boolean, borderRGB:Number):Void {
		var lineThickness:Number = drawBorder ? 0 : null;
		var lineRGB:Number = drawBorder ? (borderRGB ? borderRGB : 0x3a4352): null;
		
		drawRectangleWithBorder(clip, x, y, width, height, lineThickness, lineRGB);
	}
			
	public static function arrangeClip(clip:Object, x:Number, y:Number, width:Number, height:Number):Void {
		clip._x = x;
		clip._y = y;
		if (width != null)
			clip._width = width;
		if (height != null)
			clip._height = height;
	}
	
	public static function samePlace(clip:Object, another:Object):Void {
		clip._x = another._x;
		clip._y = another._y;
	}

	public static function removeFromArray(array:Array, item:Object):Array {
		var result:Array = new Array();
		for (var i : Number = 0; i < array.length; i++) {
			if (array[i] != item) {
				result.push(array[i]);
			}
		}
		return result;
	}
	
	public static function maxHeight(clips:Array):Number {
		var height:Number = 0;
		for (var i = 0; i < clips.length; i++) {
			if (clips[i]._height > height) {
				height = clips[i]._height;
				logger.debug("maxHeight:: " + clips[i]._name + " has current max " + height);
			}
		}
		return height;
	}
	
	public static function getDimensions(clip:MovieClip) : String {
		return "DIMENSIONS FOR clip " + clip._name +
			"    - width " + clip._width +
			"    - height " + clip._height;
	}
	
	// ==============
	// mc.drawRect() - by Ric Ewing (ric@formequalsfunction.com) - version 1.1 - 4.7.2002
	// 
	// x, y = top left corner of rect
	// w = width of rect
	// h = height of rect
	// cornerRadius = [optional] radius of rounding for corners (defaults to 0)
	// ==============
 	public static function drawRoundedRectacle(clip:MovieClip, x:Number, y:Number, w:Number, h:Number, cornerRadius:Number):Void {
		if (! cornerRadius) {
			drawRectangleNoFormatting(clip, x, y, w, h);
			return;
		}

		// init vars
		var theta, angle, cx, cy, px, py;
		// make sure that w + h are larger than 2*cornerRadius
		if (cornerRadius>Math.min(w, h)/2) {
			cornerRadius = Math.min(w, h)/2;
		}
		// theta = 45 degrees in radians
		theta = Math.PI/4;
		// draw top line
		clip.moveTo(x+cornerRadius, y);
		clip.lineTo(x+w-cornerRadius, y);
		//angle is currently 90 degrees
		angle = -Math.PI/2;
		// draw tr corner in two parts
		cx = x+w-cornerRadius+(Math.cos(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		cy = y+cornerRadius+(Math.sin(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		px = x+w-cornerRadius+(Math.cos(angle+theta)*cornerRadius);
		py = y+cornerRadius+(Math.sin(angle+theta)*cornerRadius);
		clip.curveTo(cx, cy, px, py);
		angle += theta;
		cx = x+w-cornerRadius+(Math.cos(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		cy = y+cornerRadius+(Math.sin(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		px = x+w-cornerRadius+(Math.cos(angle+theta)*cornerRadius);
		py = y+cornerRadius+(Math.sin(angle+theta)*cornerRadius);
		clip.curveTo(cx, cy, px, py);
		// draw right line
		clip.lineTo(x+w, y+h-cornerRadius);
		// draw br corner
		angle += theta;
		cx = x+w-cornerRadius+(Math.cos(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		cy = y+h-cornerRadius+(Math.sin(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		px = x+w-cornerRadius+(Math.cos(angle+theta)*cornerRadius);
		py = y+h-cornerRadius+(Math.sin(angle+theta)*cornerRadius);
		clip.curveTo(cx, cy, px, py);
		angle += theta;
		cx = x+w-cornerRadius+(Math.cos(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		cy = y+h-cornerRadius+(Math.sin(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		px = x+w-cornerRadius+(Math.cos(angle+theta)*cornerRadius);
		py = y+h-cornerRadius+(Math.sin(angle+theta)*cornerRadius);
		clip.curveTo(cx, cy, px, py);
		// draw bottom line
		clip.lineTo(x+cornerRadius, y+h);
		// draw bl corner
		angle += theta;
		cx = x+cornerRadius+(Math.cos(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		cy = y+h-cornerRadius+(Math.sin(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		px = x+cornerRadius+(Math.cos(angle+theta)*cornerRadius);
		py = y+h-cornerRadius+(Math.sin(angle+theta)*cornerRadius);
		clip.curveTo(cx, cy, px, py);
		angle += theta;
		cx = x+cornerRadius+(Math.cos(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		cy = y+h-cornerRadius+(Math.sin(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		px = x+cornerRadius+(Math.cos(angle+theta)*cornerRadius);
		py = y+h-cornerRadius+(Math.sin(angle+theta)*cornerRadius);
		clip.curveTo(cx, cy, px, py);
		// draw left line
		clip.lineTo(x, y+cornerRadius);
		// draw tl corner
		angle += theta;
		cx = x+cornerRadius+(Math.cos(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		cy = y+cornerRadius+(Math.sin(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		px = x+cornerRadius+(Math.cos(angle+theta)*cornerRadius);
		py = y+cornerRadius+(Math.sin(angle+theta)*cornerRadius);
		clip.curveTo(cx, cy, px, py);
		angle += theta;
		cx = x+cornerRadius+(Math.cos(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		cy = y+cornerRadius+(Math.sin(angle+(theta/2))*cornerRadius/Math.cos(theta/2));
		px = x+cornerRadius+(Math.cos(angle+theta)*cornerRadius);
		py = y+cornerRadius+(Math.sin(angle+theta)*cornerRadius);
		clip.curveTo(cx, cy, px, py);
	};
	
	public static function createText(parent:MovieClip, skin:Skin, id:String, text:String, wordWrap:Boolean, multiline:Boolean, selectable:Boolean, width:Number, height:Number, fontSize:Number, fonts:String):TextField {
		parent.createTextField(id, parent.getNextHighestDepth(), 0, 0, width, height);
		var field:TextField = parent[id];
		skin.initTextField(field, "left", fontSize >= 0 ? fontSize : (Stage.width >= 350 ? 12 : 10), false, fonts);
		field.wordWrap = wordWrap;
		field.multiline = multiline;
		field.selectable = selectable;
		field.textColor = 0xFFFFFF;
		if (text)
			field.text = text;
		return field;
	}
		
	public static function checkForNonNumbers(number:Number):Number {
		if (number == undefined) return 0;
		if (number == null) return 0;
		if (number == NaN) return 0;
		return number;
	} 
	

}