/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.flowlib.AbstractSlider;

class org.flowplayer.VolumeSlider extends AbstractSlider {
	
	private function getChildDescriptors():Array {
		return getConfig().getSkin().getVolumeSliderChildDescriptors();
	}

	public function addVolumeEventListener(listener:Object):Void {
		this.addEventListener("onVolumeChanged", listener);
	}

	private function arrangeSlider(mcSlider:AbstractSlider):Void {
		getConfig().getSkin().arrangeVolumeSlider(this);
	}
	
	private function onArranged():Void {
		addSeekListener(this);
		// volume to 50%
		setValue(0.5);
		removeLabel();
	}
	
	private function onCreateBackground(mcBack:MovieClip):Void {
		getConfig().getSkin().drawVolumeBarBackgroundTo(mcBack);
	}
	
	private function onCreateBorder(mcBorder:MovieClip):Void {
		var drawTrinagle:Boolean = getConfig().getSkin().drawVolumeBarBorderTo(mcBorder);
		if (! drawTrinagle) return;

		// draw volume slide triangle symbol into the border
		mcBorder.beginFill(getConfig().getSkin().getVolumeBarTriangleColor(), 100);
		mcBorder.lineStyle();
		var gap:Number = 4;
		mcBorder.moveTo(mcBorder._width - gap,  gap);
		mcBorder.lineTo(mcBorder._width - gap, mcBorder._height - gap);
		mcBorder.lineTo(gap, mcBorder._height - gap);
		mcBorder.lineTo(mcBorder._width - gap, gap);
		mcBorder.endFill();
	}

	public function onSeek(eventObject:Object):Void {
		var volume:Number = getSeekPosition();
		if (volume < 0)
			volume = 0;
		setValue(volume / 100);
		dispatchVolumeEvent(volume);
	}
	
	public function mute():Void {
		dispatchVolumeEvent(0);
	}
	
	public function unmute():Void {
		dispatchVolumeEvent();
	}
	
	private function dispatchVolumeEvent(volume:Number):Void {
		var newVolume:Number = volume == 0 ? volume : volume || getSeekPosition();
		dispatchEvent("onVolumeChanged", newVolume);
	}
	
	private function onCreateValueIndicator(mcValue:MovieClip):Void {
		getConfig().getSkin().onCreateVolumeValueMc(mcValue, this);
	}

	private function onCreateValueIndicator2(mcValue:MovieClip):Void {
		getConfig().getSkin().onCreateVolumeValue2Mc(mcValue2, this);
	}
		
	private function get barHeight():Number {
		return getConfig().getSkin().getVolumeSliderHeight();
	}
	
}
