import org.flowlib.Observable;
import org.flowplayer.ads.InVideoAd;
import org.flowplayer.config.Config;
import org.flowplayer.playlist.Clip;

import cinqetdemi.JSONConfig;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;
/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */

class org.flowplayer.ads.AdServerClient extends Observable {

	private static var AD_SERVER_VERBOSE_LOGGING:Boolean = true;
	private static var EVENT_ADS_LOADED:String = "onAdsLoaded";
	private var publisherId:String;
	private var adServerUrl:String;
	private var adServerAuth:String;
	private var countryCode:String;
	private var logger:Logger;
	private var adsLoader:JSONConfig;
	private var loadVars:LoadVars;
	private var clip:Clip;
	private var loadingFor:Array;
	
	public function AdServerClient(config:Config) {
		logger = new Logger();
		adsLoader = new JSONConfig();
		adsLoader.addConfigLoadListener(this);
		adsLoader.addLoadErrorListener(this);
		logger.addPublisher(new ConsolePublisher());
		loadingFor = new Array();
		
		loadVars = new LoadVars();
		loadVars.onData = function(data:String) {
			arguments.callee.client.onServerResponse(data);
		};
		loadVars.onLoad = function(success:Boolean) {
			arguments.callee.client.onLoad(success);
		};
		loadVars.onData.client = this;
		loadVars.onLoad.client = this;
		
		if (config) {
			this.adServerUrl = config.getAdServerUrl();
			this.adServerAuth = config.getAdServerAuth();
			this.publisherId = config.getPublisherId();
			this.countryCode = config.getCountryCode();
		}
	}
	
	public function addAdsLoaderListener(listener:Object):Void {
		addListener(EVENT_ADS_LOADED, listener);
	}
	
	private function onServerResponse(data:String):Void {
		logger.debug("received rerver response, data: " + data);
	}
	
	public function onLoad(success:Boolean):Void {
		logger.debug("onLoad: " + success);
	}
	
	private function onConfigLoaded():Void {
		loadingFor[adsLoader.fileName] = false;
		logger.info("AdServerClient:: ads receoved:");
		logger.info(adsLoader.getConfig());
		var ads:Array = InVideoAd.fromObjectsWithProps(adsLoader.getConfig());
		logger.info("parsed " + ads.length + " ads");
		if (ads.length > 0) {
			clip.setAds(ads);
			dispatchAdsAvailableEvent(clip);
		}
	}

	private function dispatchAdsAvailableEvent(clip:Clip):Void {
		dispatchEventWithObj(EVENT_ADS_LOADED, clip);
	}
	
	private function onConfigLoadError():Void {
		logger.error("failed to load ads");
	}
	
	public function loadAdsFor(clip:Clip):Void {
		if (clip.getAds()) {
			logger.debug("clip already has ads, returning");
			dispatchAdsAvailableEvent(clip);
			return;
		}
		var url:String = adsUrlFor(clip);
		if (loadingFor[url]) {
			logger.debug("ads are being loaded, returning");
			return;
		}
		
		logger.debug("loading ads");
		this.clip = clip;
		loadingFor[url] = true;
		adsLoader.fileName = url;
//		loadVars.load(url);
		adsLoader.loadConfig();
	}
	
	public function sendClickEventFor(clip:Clip, ad:InVideoAd):String {
		var url:String = commonBaseUrl("click", clip) + "&adId=" + ad.getId();
		logger.debug("sending click to " + url);
		loadVars.load(url);
		return url;
	}
	
	public function sendImpressionEventFor(clip:Clip, ad:InVideoAd):String {
		var url:String = commonBaseUrl("impression", clip) + "&adId=" + ad.getId();
		logger.debug("sending impression event to " + url);
		loadVars.load(url);
		return url;
	}
	
	function adsUrlFor(clip:Clip):String {
		var url:String = commonBaseUrl("getAds", clip);
		
		url += "&duration=" + Math.round(clip.getDuration());
		url += keywordString(clip.getKeywords());
		logger.debug("'" + url + "'");
		return url;
	}
	
	private function commonBaseUrl(action:String, clip:Clip):String {
		var url:String = adServerUrl + "/" + action + "?s=" + adServerAuth + "&videoURL=" + clip.getURL();
		url += videoKeyString(clip.getId());
		url += publisherIdString();
		url += countryCodeString();
		url += debugString();
		return url;
	}
	
	private function keywordString(keywords:Array):String {
		if (! keywords) return "";
		if (keywords.length == 0) return "";
		
		var result:String = "";
		for (var i:Number = 0; i < keywords.length; i++) {
			if (i > 0)
				result += ",";
			result += keywords[i];
		}
		return "&keyword=" + result;
	}

	private function videoKeyString(videoKey:String):String {
		if (! videoKey) return "";
		return "&videoKey=" + videoKey;
	}

	private function publisherIdString():String {
		if (! publisherId) return "";
		return "&publisherId=" + publisherId;
	}
	
	private function countryCodeString():String {
		if (! countryCode) return "";
		return "&countryCode=" + countryCode;
	}

	private function debugString() : String {
		if (! AD_SERVER_VERBOSE_LOGGING) return "";
		return "&debug=" + AD_SERVER_VERBOSE_LOGGING;
	}
	
}