/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowplayer.MediaResizer;
import org.flowplayer.MediaContext;
import org.flowplayer.MediaControl;
import org.flowplayer.PlaybackStatus;
import org.flowplayer.playlist.Clip;
import org.flowplayer.playlist.PlayList;
import LuminicBox.Log.Logger;
import LuminicBox.Log.ConsolePublisher;
import org.flowlib.Observable;
import org.flowplayer.image.ImageView;
import org.flowplayer.image.LayeredImagesView;
import org.flowlib.AlphaCycleEffect;

/**
 * @author Anssi
 */
class org.flowplayer.audio.MP3Controller extends Observable implements MediaControl {
	
	public static var logger:Logger;
	private var sound:Sound;
	private var pausedPosition:Number;
	private var paused:Boolean;
	private var playing:Boolean;
	private var clip:Clip;
	private var durationSeconds:Number;
	private var durationCalculated:Boolean;
	private var imageView:LayeredImagesView;
	private var fadeInEffect : AlphaCycleEffect;
	private var fadeOutEffect : AlphaCycleEffect;

	public function MP3Controller() {
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
	}
	
	public function setSound(sound:Sound):Void {
		this.sound = sound;
		
		var onId3Listener:Function = function() {
			MP3Controller.logger.debug("onID3, " + arguments.callee.controller);
			
			var durationSecs:Number = arguments.callee.controller.estimateDuration(this);
			MP3Controller.logger.debug("duration " + durationSecs);
			
			MP3Controller(arguments.callee.controller).onID3();
		}; 
		onId3Listener.controller = this;
		this.sound.onID3 = onId3Listener;

		var onLoadedListener:Function = function(success:Boolean) {
			MP3Controller(arguments.callee.controller).onMp3Loaded(success);			
		};
		onLoadedListener.controller = this;
		this.sound.onLoad = onLoadedListener;
		
		var onSoundCompleteListener:Function = function () {
			MP3Controller(arguments.callee.controller).onSoundComplete();			
		};
		onSoundCompleteListener.controller = this;
		this.sound.onSoundComplete = onSoundCompleteListener;
	}
	
	public function estimateDuration(sound:Sound):Number {
		var durationSecs:Number = (sound.duration/(sound.getBytesLoaded()/sound.getBytesTotal()))/1000;
		return durationSecs;
	}
	
	public function onID3():Void {
		logger.debug("sound.onID3");
		for (var prop:String in sound.id3) {
			logger.debug(prop + ": " + sound.id3[prop]);
		}
		var duration:Number = sound.ID3.TLEN;
		if (duration > 0) {
			setDuration(duration);
		} else {
			duration = estimateDuration(sound);
			if (duration)
				durationSeconds = duration;
		}
		
		logger.debug("sound.ID3.TLEN " + sound.ID3.TLEN);
		logger.debug("sound.ID3.TIME " + sound.ID3.TIME);
		logger.debug("sound.id3.TLEN " + sound.id3.TLEN);
		logger.debug("sound.id3.TIME " + sound.id3.TIME);
	}
	
	public function onLoaded():Void {
		dispatchEventWithObj("onLoadBegin", clip);
		dispatchEventWithObj("onBufferFull", clip);
	}

	private function onMp3Loaded(success:Boolean):Void {
		logger.debug("mp3 loaded fully");
		setDuration(sound.duration / 1000);
		durationCalculated = true;
	}

	private function onSoundComplete() : Void {
		dispatchEventWithObj("onClipDone", clip);
	}

	public function startBuffering(clip:Clip):Void {
		this.clip = clip;
		dispatchEventWithObj("onStartBuffering", clip);
		showOverlayImage();
		durationCalculated = false;
		pausedPosition = 0;
		sound.start(0);
		sound.loadSound(clip.getURL(), true);
		if (! playing) {
			logger.info("MP3Controller: not playing yet (only buffering), pausing playback");
			paused = true;
			sound.stop();
		}
	}

	public function play(clip:Clip):Void {
		// TODO: add protection code support
		logger.debug("starting to play audio from " + clip.getURL());
		playing = true;
		paused = false;
		
				// this needs to be here because it is possible to seek before starting the playback
		if (paused) {
			resume();
			this.paused = false;
			return;
		}
		
		startBuffering(clip);
	}

	public function stop():Void {
		sound.stop();
		resetState();
	}

	public function stopCurrent():Void {
		stop();
	}

	public function pause():Void {
		logger.debug("MP3Controller.pause at position " + sound.position + " id3 " + sound.id3);
		pausedPosition = sound.position;
		this.paused = true;
		sound.stop();
	}

	public function resume():Void {
		if (! this.paused) {
			logger.debug("MP3Controller: not paused calling play()");
			play(clip);
		} else {
			logger.debug("MP3Controller.resume at position " + sound.position);
			this.paused = false;
			sound.start(pausedPosition / 1000);
		}
		this.playing = true;
//		dispatchEventWithObj("onLoadBegin", clip);		
	}

	public function seek(percentage:Number):Void {
		seekToTime(getDuration() * (percentage / 100));
	}

	public function seekToTime(offset:Number):Void {
		sound.start(offset);
	}

	public function attachView(mcDisplay:LayeredImagesView, mediaType:String):Void {
		this.imageView = mcDisplay;
		imageView.addLoadListener(this);
	}

	public function attachVideo(video:Video, mediaType:String):Void {
	}

	public function setContext(context:MediaContext) {
	}

	public function isPaused():Boolean {
		return paused;
	}

	public function isPlaying():Boolean {
		return playing;
	}

	public function getPlaybackStatus():PlaybackStatus {
		return new PlaybackStatus(getDuration(), sound.position / 1000, sound.getBytesTotal(), sound.getBytesLoaded());
	}

	public function onClipChanged():Void {
	}

	public function reset():Void {
		stop();
	}

	public function addCuePoint(cueTimeSeconds:Number, cueName:String, cueParams:Object):Void {
		// TODO: add cue point support
	}

	public function addCuePoints(cuePoints:Array):Void {
	}

	public function setPlayList(playList:PlayList):Void {
	}

	public function playCurrent():Void {
		throw new Error("Unsupported method!");
	}

	public function toggleLoop():Void {
	}

	public function isLooping():Boolean {
		return false;
	}

	private function setDuration(duration:Number):Void {
		this.durationSeconds = duration;
		this.durationCalculated = true;
	}

	private function getDuration():Number {
		if (durationCalculated)
			return durationSeconds;
		else
			return estimateDuration(sound);
	}

	private function resetState():Void {
		playing = false;
		paused = false;
		durationCalculated = false;
		durationSeconds = 0;
	}

	private function showOverlayImage():Void {
		var overlayFile:String = clip.getOverlayFileName();
		if (overlayFile) {
			logger.debug("showing mp3 overlay art " + overlayFile + " on " + imageView);
			imageView.load(undefined, false, overlayFile, false);
		} else {
			onLoaded();
		}
	}

	public function setViewVisibility():Boolean {
		if (clip.getOverlayFileName()) {
			logger.error("MP3 controller showing image view");
			AlphaCycleEffect.fade(imageView, true);
		} else if (imageView._visible) {
			AlphaCycleEffect.fade(imageView, false);
		}
		return true;
	}

	public function toString():String {
		return "MP3Controller";
	}

	public function allowCenterViewOnLoadBegin() : Boolean {
		return false;
	}
	
	public function rewind():Void {
		logger.debug("MP3Controller::rewind()");
		if (! playing) {
			stop();
			return;
		}
		seek(0);
		pause();
		playing = false;
	}
}