/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.AlphaCycleEffect;
import org.flowlib.ColorBrightener;
import org.flowlib.AbstractColorTransform;
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.ClipDescriptor;
import org.flowplayer.Util;
import org.flowplayer.image.ResizableView;

/**
 * @author Anssi
 */
class org.flowplayer.button.OverlayPlayButton extends NewAbstractMovieClip implements ResizableView {
	
	private var mcBackground:MovieClip;
	private var mcFace:MovieClip;
	private var maxWidth:Number;
	private var maxHeight:Number;
	private var mouseHoverTransform:AbstractColorTransform;
	
	
	private function getChildDescriptors():Array {
		return [new ClipDescriptor("PlayButtonOverlayBase", "mcBackground"), new ClipDescriptor("PlayButtonOverlayTriangle", "mcFace")];		
	}

	private function allChildrenCreated(clips:Array):Void {
		mcBackground = clips[0];
		mcFace = clips[1];
		mouseHoverTransform = new ColorBrightener();
		mouseHoverTransform.addClip(mcFace);
		mouseHoverTransform.addClip(mcBackground);
		this._parent.hitArea = mcFace;
		
		this.onRollOver = function():Void {
			OverlayPlayButton(this).mouseOver(true);
		};
		this.onRollOut = function():Void {
			OverlayPlayButton(this).mouseOver(false);
		};
		this.onMouseUp = function():Void {
			OverlayPlayButton(this).clicked();
		}
	}
	
	public function clicked():Void {
		if (isMouseOverClip(this))
			AlphaCycleEffect.fade(mcFace, false, 3);
	}

	public function mouseOver(isMouseOver:Boolean):Void {
		NewAbstractMovieClip.logger.debug(this._name + ", switchFaces()");
		
		if (isMouseOver)
			mouseHoverTransform.applyTransform();
		else
			mouseHoverTransform.applyNormal();
	}

	function arrange():Void {
		if (! (mcFace)) return;
		logger.error("arrange on OverlayPlayButton");
		mcBackground._width = __width;
		mcBackground._height = __height;
		
		mcFace._width = __width;
		mcFace._height = __height;
		
		Util.centerClip(this, maxWidth, maxHeight);
		mcFace._visible = true;
		mouseHoverTransform.applyNormal();
	}

	function setMaxSize(width:Number, height:Number):Void {
		logger.error("setMaxSize on OverlayPlayButton");
		this.maxWidth = width;
		this.maxHeight = height;
		arrange();
	}

	function resizeTo(sizingOption:Number):Void {
		arrange();
	}

	function loadImage(imageURL:String, allowResize:Boolean):Void {
	}

	function hide() : Void {
	}
	
	public function playClicked():Void {
		AlphaCycleEffect.fade(mcFace, false, 3);
	}
}