﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.flowplayer.config.Config;
import org.flowplayer.playlist.PlayList;
import org.flowplayer.playlist.Clip;
import org.flowplayer.skin.Skin;
import org.flowplayer.MediaSize;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;

/**
 * @author anssi
 */
class org.flowplayer.config.DefaultsConfig implements Config {
	private static var instance:DefaultsConfig;
	public static var logger:Logger;
	
	private function getSkinID():String {
		return TextField(_root.skin).text;
	}
	
	public static function getInstance():DefaultsConfig {
		if (! instance)
			instance = new DefaultsConfig();
		return instance;
	}
	
	private function DefaultsConfig() {
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
	}

 	public function getConfigFileName():String {
 		return undefined;
 	}
  	
  	public function setConfigFileName(fileName:String):Void {
  	}

	public function getBaseURL():String {
		return undefined;
	}
		
	public function setBaseURL(baseURL:String) {
	}
	
	public function isAutoBuffering():Boolean {
		logger.info("Using default autoBuffering = true");
		return true;
	}
	
	public function isAutoPlay():Boolean {
		logger.info("Using default autoPlay = true");
		return true;
	}
 	
 	public function isLooping():Boolean {
 		return true;
 	}
 	
 	public function getStartingBufferLength():Number {
 		return undefined;
 	}
 	
 	public function getMainBufferLength():Number {
 		return 10;
 	}
	
	public function getVideoHeight() : Number {
		return undefined;
	}
 	
 	public function showControls():Boolean {
		return true;
 	}

	public function showPlayListButtons() : Boolean {
		return false;
	}
	
	public function showPlayButton():Boolean {
		return true;
	}
	
	public function showMuteVolumeButton():Boolean {
		return true;
	}

	public function useEmbeddedButtonImages():Boolean {
		return true;
	}
			
 	public function getSkin(rootConfig:Config):Skin {
 		return Skin.create(rootConfig, getSkinID());
 	}

	public function getVideoFile(useBaseUrl:Boolean):String {
		return undefined;
	}
 	
	public function getNoVideoClip():Clip {
		return undefined;
	}
 	
 	public function getPlayList(baseURL:String):PlayList {
 		return undefined;
 	}

	public function getSkinImagesBaseURL() : String {
		return undefined;
	}

	public function getSplashImageFile() {
		return undefined;
	}

	public function scaleSplash() : Boolean {
		return false;
	}

	public function thumbsOnFLV() : Boolean {
		return false;
	}

	public function getStreamingServerURL() : String {
		return undefined;
	}

	public function getThumbnails() : Array {
		return undefined;
	}

	public function useConfigInject():Boolean {
		return false;
	}

	public function getStreamingServer() : String {
		return undefined;
	}
	
	public function getInitialMediaSize():Number {
		return MediaSize.ORIGINAL;
	}
	
	public function setIninitalMediaSize(newSize:Number):Void {
	}
	
	
	public function getProtectionFileName():String {
		return "flowplayer_auth.txt";
	}
	
	public function showMenu():Boolean {
		return true;
	}

 	/*
	 * Methods for skinning:
	 */
	 
 	public function getProgressIndicatorColor1():Number {
		return undefined;
 	}
 	
 	public function getProgressIndicatorColor2():Number {
		return undefined;
 	}
 	
 	public function getBufferIndicatorColor1():Number {
		return undefined;
 	}
 	
 	public function getBufferIndicatorColor2():Number {
		return undefined;
 	}
 	
 	public function getProgressBarBorderColor1():Number {
		return undefined;
 	}
 	
 	public function getProgressBarBorderColor2():Number {
		return undefined;
 	}
	
	public function getBufferingAnimationColor():Number {
		return undefined;
 	}

	public function getControlBarBackgroundColor():Number {
		return undefined;
	}
	
	public function getTimeDisplayFontColor():Number {
		return undefined;
	}
	
	public function getProgressBarHeight():Number {
		return undefined;
	}
	
	public function getProgressBarAreaHeight():Number {
		return undefined;
	}

	public function stringify() : String {
		return null;
	}
	
	public function showMenuItem(itemNumber:Number):Boolean {
		return true;
	}

	public function getFullScreenScriptURL() : String {
		return "http://flowplayer.org/video/2_1/fullscreen.js";
	}

	public function showFullScreenButton() : Boolean {
		return true;
	}

	public function isAutoRewind() : Boolean {
		return false;
	}

	public function getControlsWidth() : Number {
		return Stage.width;
	}

	public function setControlsWidth(newWidth : Number) : Void {
	}

	public function isFullScreen():Boolean {
		return false;
	}

	public function setFullScreen(fullScreen:Boolean):Void {
	}

	public function getEmbedWidth():Number {
		return Stage.width;
	}

	public function getEmbedHeight():Number {
		return Stage.height;
	}

	public function setEmbedWidth(width:Number):Void {
	}

	public function setEmbedHeight(height:Number):Void {
	}

	public function setIsEmbedded(embedded:Boolean):Void {
	}

	public function isEmbedded():Boolean {
		return null;
	}

	public function getProperty(propertyName:String):Object {
		return null;
	}

	function useFlashNativeFullScreen():Boolean {
		return true;
	}
	
	public function useHwScaling():Boolean {
		return false;
	}

	public function useSmoothing():Boolean {
		return true;
	}
	
	public function getRtmpConfigUrl() : String {
		return undefined;
	}

	public function getRtmpConfigParam():Object {
		return undefined;
	}

	public function removeChildConfig() : Void {
	}

	public function setPlayList(playList : PlayList) : Void {
	}

	public function getGoogleAnalyticsPrefix():String {
		return undefined;
	}

	public function getEmbedCode():String {
		return undefined;
	}
	
	public function getAdServerUrl():String {
		return "http://piimaa.info/flowplayer/api";
	}

	public function getAdServerAuth():String {
		return "foo";
	}

	public function getPublisherId():String {
		return undefined;
	}

	public function getCountryCode():String {
		return undefined;
	}

	public function getEmailPostUrl() : String {
		return "http://community.flowplayer.org/video/2_1/sendmail.php";
	}

	public function getVideoLink() : String {
		return undefined;
	}

	public function showStopButton() : Boolean {
		return false;
	}

	public function getEmailVideoLink() : String {
		return undefined;
	}

	public function getEmailDefaultMessage() : String {
		return undefined;
	}

	public function showVolumeSlider() : Boolean {
		return true;
	}
	
	public function enableAds():Boolean {
		return false;
	}
	
	public function showScrubber():Boolean {
		return true;
	}
	
	public function usePlayOverlay():Boolean {
		return true;
	}
	
	public function controlsOverVideo(isFullScreen:Boolean):Boolean {
		return isFullScreen;
	}

	public function lockControls():Boolean {
		return false;
	}
	
	public function getControlBarGloss():String {
		return "low";
	}
	
	public function getWatermarkUrl():String {
		return undefined;
	}
	
	public function showWatermark():String {
		return "fullscreen";
	}
	
	public function getWatermarkLinkUrl():String {
		return undefined;
	}
}
