import org.flowlib.Observable;

import cinqetdemi.JSON;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;
/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
class org.flowplayer.config.RTMPConfigLoader extends Observable {
	
	public static var EVENT_CONFIG_LOADED:String = "onRtmpConfigLoaded";
	public static var EVENT_CONFIG_LOAD_ERROR:String = "onRtmpConfigLoadError";

	private var connection:NetConnection;
	private static var logger:Logger;
	private var getConfigParam:Object;
	
	public function RTMPConfigLoader() {
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
	}
	
	public function load(rtmpConfigUrl:String, getConfigParam:Object):Void {
		this.getConfigParam = getConfigParam instanceof String ? getConfigParam : JSON.stringify(getConfigParam);
		initConnection(rtmpConfigUrl);
	}
	
	private function initConnection(url:String):Void {
		connection = new NetConnection();
		connectionStatus.loader = this;
		onResult.loader = this;
		onStatus.loader = this;
		connection.onStatus = connectionStatus;
		connection.connect(url);
	}
	
	private function disconnect():Void {
		connection.close();
	}

	private function connectionStatus(info:Object):Void {
		logger.debug("connectionStatus " + info.code);
		if (info.code == "NetConnection.Connect.Success") {
			logger.debug("connection success");
			arguments.callee.loader.loadConfig();
		} else if (info.code == "NetConnection.Connect.Failed") {
			logger.error("Failed to connect to the media server! Unable to load config.");
			arguments.callee.loader.dispatchEventWithObj(RTMPConfigLoader.EVENT_CONFIG_LOAD_ERROR, info);
		}
	}
	private function loadConfig():Void {
		logger.debug("calling remote getStreamPlayerConfig with param '" + getConfigParam + "'");
		connection.call("getStreamPlayerConfig", this, getConfigParam);
	}

	private function onResult(result:Object):Void {
		logger.debug("onResult: " + result);
		arguments.callee.loader.dispatchEventWithObj(RTMPConfigLoader.EVENT_CONFIG_LOADED, result);
		arguments.callee.loader.disconnect();
	}

	private function onStatus(status:Object):Void {
		logger.debug("onStatus: " + status);
	}

}