/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.skin.Skin;
import org.flowplayer.email.MailSender;
import org.flowplayer.playlist.PlayList;
import org.flowplayer.Util;
import org.flowlib.AlphaCycleEffect;
import org.flowlib.DelayedFade;

/**
 * @author Anssi
 */
class org.flowplayer.email.EmailView extends NewAbstractMovieClip {
	private static var EVENT_DONE:String = "onEmailSendingComplete";
	private var mcBackground:MovieClip;
	private var txtTitle:TextField;
	private var txtFromHeader:TextField;
	private var txtFrom:TextField;
	private var txtToHeader:TextField;
	private var txtTo:TextField;
	private var txtMessageHeader:TextField;
	private var txtMessage:TextField;
	private var txtToExtraHeader;
	private var txtStatus:TextField;
	private var txtErrorStatus:TextField;
	private var btnPost:MovieClip;
	private var btnCancel:MovieClip;
	private var mailSender:MailSender;
	private var videoLink:String;
	private var btnClose:MovieClip;
	private static var LABEL_WIDTH:Number = 90;
	private static var FIELD_WIDTH:Number = 200;

	public function EmailView() {
		super();
	}

	public function addEmailSendingCompleteListener(listener:Object):Void {
		addEventListener(EVENT_DONE, listener);
	}
	
	private function dispatchEmailSendingCompleteEvent(wasSent:Boolean):Void {
		dispatchEvent(EVENT_DONE, wasSent);
	}
	
	public function setSender(mailSender:MailSender):Void {
		this.mailSender = mailSender;
		this.mailSender.addMailSendListener(this);
	}
	
	public function setVideoLink(videoLink:String):Void {
		this.videoLink = videoLink;
		txtMessage.text = getDefaultMessageText();
	}
	
	function arrange():Void {
		logger.debug("EmailView.arrange");
		btnClose._x = __width - btnClose._width - 9;
		btnClose._y = 5;

		var margin:Number = 30;
		var gap:Number = 10;
		
		txtTitle._x = margin;
		txtTitle._y = 20;
		txtTitle._width = __width - 2*margin;

		arrangeTextFieldsVertically(txtFromHeader, txtFrom, txtTitle, margin, gap);
		arrangeTextFieldsVertically(txtToHeader, txtTo, txtFrom, margin, gap);
		arrangeTextFieldsVertically(txtToExtraHeader, undefined, txtTo, margin, gap);

		txtMessageHeader._x = margin;
		txtMessageHeader._y = txtToExtraHeader._y + txtToExtraHeader._height + 2*gap;
		txtMessage._x = txtMessageHeader._x;
		txtMessage._y = txtMessageHeader._y + txtMessageHeader._height + gap;
		txtMessage._width = __width - 2*margin;
		
		btnPost._x = txtMessage._x + txtMessage._width - btnPost._width;
		btnPost._y = txtMessage._y + txtMessage._height + gap;
		btnCancel._x = btnPost._x - btnCancel._width - gap;
		btnCancel._y = btnPost._y;
		
		txtStatus._x = margin;
		txtStatus._y = txtMessage._y + txtMessage._height + gap;
		Util.samePlace(txtErrorStatus, txtStatus);
		
		mcBackground._width = __width;
		mcBackground._height = txtStatus._y + txtStatus._height + gap;
	}
		
	private function createChildren():Void {
		createBackground();
		createMainTitle();
		createFromHeader();
		createFromField();
		createToHeader();
		createToField();
		createToExtraHeader();
		createMessageHeader();
		createMessageField();
		createStatusField();
		createErrorStatusField();
		createCancelButton();
		createPostButton();
		createCloseButton();
		
		var index:Number = 1;
		txtTitle.tabIndex = index++;
		txtFrom.tabIndex = index++;
		txtTo.tabIndex = index++;
		txtMessage.tabIndex = index++;
		btnPost.tabIndex = index++;
		btnCancel.tabIndex = index++;
		btnClose.tabIndex = index++;
	}

	private function createCloseButton():Void {
		btnClose = getSkin().createEmailCloseButton(this);
		btnClose.onRelease = function() {
			this._parent.close();
		};
	}

	private function createBackground() : Void {
		getSkin().createEmailViewBackground("mcBackground", this, 100, 50);	
	}

	private function createMainTitle():Void {
		getSkin().createEmailViewMainTitle(this, "txtTitle", Stage.width, 25); 
	}

	private function createFromHeader() : Void {
		getSkin().createEmailViewFromLabel(this, "txtFromHeader", LABEL_WIDTH, 20); 
	}

	private function createMessageHeader() : Void {
		getSkin().createEmailViewMessageBodyLabel(this, "txtMessageHeader", LABEL_WIDTH, 20); 
	}

	private function createFromField() : Void {
		createText("txtFrom", "", false, false, true, FIELD_WIDTH, 20);
		makeEditable(txtFrom);
	}

	private function createToHeader():Void {
		getSkin().createEmailViewToLabel(this, "txtToHeader", LABEL_WIDTH, 20); 
	}

	private function createToField():Void {
		createText("txtTo", "", false, false, true, FIELD_WIDTH, 20);
		makeEditable(txtTo);		
	}
	
	private function createToExtraHeader():Void {
		getSkin().createEmailViewToExtraLabel(this, "txtToExtraHeader", 300, 20); 
	}

	private function createMessageField():Void {
		createText("txtMessage", "", true, true, true, 200, 80);
		makeEditable(txtMessage);
	}

	private function createStatusField():Void {
		createText("txtStatus", "", true, true, false, 200, 20);
		txtStatus._visible = false;
		txtStatus.textColor = 0x00FF00;
	}

	private function createErrorStatusField():Void {
		createText("txtErrorStatus", "", true, true, false, 200, 40);
		txtErrorStatus._visible = false;
		txtErrorStatus.textColor = 0xFF0000;
	}

	private function createCancelButton() : Void {
		btnCancel = getSkin().createEmailCancelButton(this);
		btnCancel.onRelease = function() {
			arguments.callee.view.close();
		};
		btnCancel.onRelease.view = this;	
	}

	private function createPostButton() : Void {
		btnPost = getSkin().createEmailPostButton(this);
		btnPost.onRelease = function() {
			arguments.callee.view.sendMail();
		};
		btnPost.onRelease.view = this;	
	}
	
	private function sendMail():Void {
		if (! validateField(txtFrom)) return;
		if (! validateField(txtTo)) return;
		if (! validateField(txtMessage)) return;
		showStatus(getSkin().getEmailSendInProgressMessage("Sending..."));
		txtMessage.text = replaceTag(txtMessage.text, "[from]", txtFrom.text);
		mailSender.send(txtTo.text, txtFrom.text, txtMessage.text, videoLink);
//		dispatchEmailSendingCompleteEvent(true);
	}
	
	private function onMailSent(success:Object):Void {
		if (success) {
			showStatus(getSkin().getEmailSendSuccessMessage("E-mail was successfully sent!"));
			txtTo.text = "";
		} else
			showError(getSkin().getEmailSendFailedMessage("E-mail sending failed!"));
	}
	
	private function showStatus(text:String, createHideTimeout:Boolean):Void {
		AlphaCycleEffect.fade(txtErrorStatus, false);
		doShowStatus(txtStatus, text, createHideTimeout);
	}
	
	private function showError(text:String, createHideTimeout:Boolean):Void {
		AlphaCycleEffect.fade(txtStatus, false);
		doShowStatus(txtErrorStatus, text, createHideTimeout);
	}
	
	private function doShowStatus(inField:TextField, text:String, createHideTimeout:Boolean):Void {
		inField.text = text;
		AlphaCycleEffect.fade(inField, true);
		if (createHideTimeout) {
			DelayedFade.execute(inField, false, 3000);
		}
	} 
	
	private function close():Void {
		txtStatus.text = "";
		dispatchEmailSendingCompleteEvent(false);
	}

	private function validateField(field:TextField):Boolean {
		if (! field.text) {
			Selection.setFocus(field);
			showError(getSkin().getEmailSendMandatoryFieldMissing("Please fill in all values in the form."), true);
			return false;
		}
		return true;
	}

	private function makeEditable(field:TextField):Void {
		field.type = "input";
		field.border = false;
		field.background = true;
		field.textColor = 0;
	}

	private function getDefaultMessageText():String {
		var defaultMsg:String = getConfig().getEmailDefaultMessage();
		if (! defaultMsg) return "";
		return replaceTag(defaultMsg, "[video_link]", videoLink);
	}
	
	private function replaceTag(text:String, tag:String, replaceText:String):String {
		var index:Number = text.indexOf(tag); 
		if (index >= 0) {
			return text.substring(0, index) + replaceText + text.substring(index + tag.length, text.length);
		}
		return text;
	}

}