import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;
import org.flowlib.Observable;
/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
class org.flowplayer.email.MailSender extends Observable {
	private var EVENT_MAIL_SENDING_DONE:String = "onMailSent";
	private var postUrl:String;
	private var sender:LoadVars;
	private var receiver:LoadVars;
	private var logger:Logger;
	private var sendInterval : Number;
	
	public function MailSender(postUrl:String) {
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
		
		this.postUrl = postUrl;
		this.sender = new LoadVars();
		this.receiver = new LoadVars();
		
		receiver.onData = function(data:String) {
			arguments.callee.logger.debug("MailSender onData: " + data);
			arguments.callee.parent.deleteTimeOutInterval();			
			arguments.callee.parent.dispatchEmailDoneEvent(data && (data.indexOf("OK") >= 0));
		};
		receiver.onData.logger = logger;
		receiver.onData.parent = this;
	}
	
	public function addMailSendListener(listener:Object):Void {
		addListener(EVENT_MAIL_SENDING_DONE, listener);
	}
	
	private function dispatchEmailDoneEvent(success:Boolean):Void {
		dispatchEventWithObj(EVENT_MAIL_SENDING_DONE, success);
	}
	
	private function onSendTimeout():Void {
		deleteTimeOutInterval();
		dispatchEmailDoneEvent(false);
	}
	
	private function deleteTimeOutInterval():Void {
		clearInterval(sendInterval);
	}
	
	public function send(toAddress:String, from:String, body:String, videoLink:String):Void {
		sender.toAddress = toAddress;
		sender.from = from;
		sender.body = body;
		sender.video = videoLink;
		logger.debug("sending mail to " + postUrl);
		logger.debug("toAddress: " + sender.toAddress);
		logger.debug("from: " + sender.from);
		logger.debug("body: " + sender.body);
		logger.debug("video: " + sender.video);
		sender.sendAndLoad(postUrl, receiver, "POST");
		sendInterval = setInterval(this, "onSendTimeout", 5000);
	}
	
}