import org.flowplayer.ClipDescriptor;
import org.flowplayer.menu.FlowMenuItem;
import org.flowplayer.menu.MenuItemBorder;
import org.flowlib.NewAbstractMovieClip;
/**
 * @author anssi
 */
class org.flowplayer.menu.FlowMenu extends NewAbstractMovieClip {
	
	public static var EVENT_SELECTED:String = "onMenuSelected";
	
	private var menuId:Number;
	private var items:Array;	
	private var itemClips:Array;
	
	private static var MENU_ITEM_HEIGHT = 18;
	private static var MENU_ITEM_WIDTH = 140;
	
	public function FlowMenu() {
		this.menuId = MenuItemBorder.getNextMenuId();
		items = new Array();
		super();
	}
	
	public function addSelectionListener(listener:Object):Void {
		addEventListener(EVENT_SELECTED, listener);
	}
	
	public function addActionItem(labelText:String, menuItemIndex:Number, disableWhenSelected:Boolean):FlowMenuItem {
		logger.error("adding menu item, " + disableWhenSelected);
		return addItem(FlowMenuItem.createActionItem(labelText, menuItemIndex, disableWhenSelected));
	}
	
	public function hasItem(menuItemIndex:Number):Boolean {
		for (var i:Number = 0; i < items.length; i++) {
			var item:FlowMenuItem = items[i];
			logger.debug("MENU.hasItem:");
			logger.debug(menuItemIndex);
			logger.debug(item.object);
			if (Number(item.object) == menuItemIndex)
				return true;
		}
		return false;
	}
	
	public function addSelectableItem(labelText:String, menuItemIndex:Number, makeSelected:Boolean):FlowMenuItem {
		return addItem(FlowMenuItem.createSelectableItem(labelText, menuItemIndex, makeSelected));
	}
	
	private function addItem(item:FlowMenuItem):FlowMenuItem {
		items.push(item);
		return item;
	}
	
	public function fireSelectionEvent():Void {
		dispatchEvent(EVENT_SELECTED, MovieClip(MenuItemBorder.getSelectedItem(menuId)).item);
	}

	public function hasSelection():Boolean {
		return MenuItemBorder.getSelectedItem(menuId) != undefined;
	}

	public function selectItem(menuItemIndex:Number):Void {
		for (var i:Number = 0; i < itemClips.length; i++) {
			var item:FlowMenuItem = FlowMenuItem(itemClips[i].item);
			if (item.object == menuItemIndex) {
				itemClips[i].selected();
				return;
			}
		}
	}
	
	public function getItems():Array {
		return items;
	}
	
	public function getItemClips():Array {
		return itemClips;
	}
	
	public function show():Array {
		var clips:Array = new Array();
		for (var i : Number = 0; i < items.length; i++) {
			clips.push(new ClipDescriptor("MenuItemBorder", "border" + i, null));
		}
		loadClips(clips);
		return itemClips;
	}
	
	public function enableAllItems():Void {
		for (var i:Number = 0; i < itemClips.length; i++) {
			var clip:MovieClip = MovieClip(itemClips[i]);
			clip.enabled = true;
			clip.brighten();
		}
	}
	
	public function disableItem(item:FlowMenuItem):Void {
		enable(item, false);
	}
	
	public function enableItem(item:FlowMenuItem):Void {
		enable(item, true);
	}
	
	private function enable(item:FlowMenuItem, enabled:Boolean):Void {
		for (var i : Number = 0; i < itemClips.length; i++) {
			var clip:MovieClip = itemClips[i];
			if (clip.item == item) {
				clip.enabled = enabled;
				if (enabled)
					clip.brighten();
				else
					clip.dim();
			}
		}
	}
	
	public function allChildrenCreated(clips:Array):Void {
		itemClips = clips;
		for (var i : Number = 0; i < clips.length; i++) {
			initMenuItemClip(MenuItemBorder(clips[i]), items[i], i);
		}
		beginFill(0x777777, 100);		
		drawRectangle(this, 0, 0, MENU_ITEM_WIDTH + 1, this._height, true);
		endFill();
	}
	
	private function initMenuItemClip(clip:MenuItemBorder, item:FlowMenuItem, itemNo:Number):Void {
		clip.addEventListener(MenuItemBorder.EVENT_CLICKED, this);
		MovieClip(clip).item = item;

		clip.setSize(MENU_ITEM_WIDTH, MENU_ITEM_HEIGHT);
		clip.fill(0x000000, MENU_ITEM_WIDTH, MENU_ITEM_HEIGHT, getSkin().getMenuItemBorderAlpha());
		clip.disableSelectedEffect();
		clip.setMenuId(menuId);
		if (item.isSelected) {
			clip.selected();
		}
		
		createTextLabel(clip, item);
		clip._height = MENU_ITEM_HEIGHT;
		clip._y = itemNo * MENU_ITEM_HEIGHT + 1;
		clip._x = 1;
	}
	
	private function createTextLabel(parent:MovieClip, item:FlowMenuItem) : Void {
		logger.debug("createTextLabel on " + parent);
		parent.createTextField("text", parent.getNextHighestDepth(), 4, 0, MENU_ITEM_WIDTH, MENU_ITEM_HEIGHT);
		var field:TextField = parent.text;
		field.text = item.name;
		logger.debug("FlowMenu, initialized menu item text " + parent.text + " from item " + item);
		var textFormat:TextFormat = getDefaultTextFormat();
		textFormat.italic = false;
		textFormat.bold = true;
		textFormat.font = "_sans";
		field.setTextFormat(textFormat);
//		field.antiAliasType = "advanced";
		field.textColor = 0xFFFFFF;
	}

	private function arrange():Void {
	}
	
		
	public function onClicked(clip:MovieClip):Void {
		dispatchEvent(EVENT_SELECTED, clip.item);
		this._visible = false;
	}
	
}