/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.AlphaCycleEffect;
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.menu.FlowMenuItem;
import org.flowlib.ColorFader;

class org.flowplayer.menu.MenuItemBorder extends NewAbstractMovieClip {
	
	public static var EVENT_CLICKED:String = "onClicked";
	
 	private var mcHighlight:MovieClip;
 	public var mcSelectHighlight:MovieClip;

	private var effect:AlphaCycleEffect;
	private var selectedEffect:AlphaCycleEffect;
	private var useSelectedEffect:Boolean;
	
	private var menuId:Number;
	private static var nextMenuId = 0;
 	private static var selectedItem:Array = new Array();
 	private var fader:ColorFader;

	public function MenuItemBorder() {
		fader = new ColorFader();
		fader.addClip(this);
	}
	
	public function dim():Void {
		fader.dim();
	}
	
	public function brighten():Void {
		fader.brighten();
	}

	public function setMenuId(menuId:Number) {
		this.menuId = menuId;
	}
	
	public static function getNextMenuId():Number {
		return nextMenuId++;
	}
	
	function arrange():Void {
		mcHighlight._width = __width;
		mcHighlight._height = __height;
		mcSelectHighlight._width = __width;
		mcSelectHighlight._height = __height;
		mcHighlight._x = 0;
		mcSelectHighlight._x = 0;
	}
	
	public function fill(color:Number, width:Number, height:Number, alpha:Number):Void {
		clear();
		beginFill(color, alpha);
		drawRectangle(this, 0, 0, width, height, false);
		endFill();
	}
	
	public function disableSelectedEffect():Void {
		useSelectedEffect = false;
	}
	
	private function createChildren():Void {
		createEmptyMovieClip("mcHighlight", getNextHighestDepth());
		drawRectangle(mcHighlight, 0, 0, 100, 20, true);
		mcHighlight._visible = false;
		
		createEmptyMovieClip("mcSelectHighlight", getNextHighestDepth());
		mcSelectHighlight.beginFill(0xAAFFAA, 50);
		drawRectangle(mcSelectHighlight, 0, 0, 100, 20, false);
		mcSelectHighlight.endFill();
		
		mcSelectHighlight._visible = false;
		mcSelectHighlight.alphaChange = -20;
		
		effect = new AlphaCycleEffect(this);
		selectedEffect = new AlphaCycleEffect(mcSelectHighlight);
		useSelectedEffect = true;
	}
	
	private function allChildrenCreated(clips:Array):Void {
		this.onRollOver = function():Void {
			this.rollOverEfect();
		};
		this.onRollOut = function():Void {
			this.rollOutEfect();
		};
		this.onRelease = function():Void {
			this.selected();
		};
 	}
	
	public function rollOverEfect():Void {
		mcHighlight._visible = true;
		effect.executeOneCycle();
					
		if (! isMouseOverClip(this)) {
			rollOutEfect();
		}
	}
	
	public function rollOutEfect():Void {
		mcHighlight._visible = false;
	}
	
	public function terminateSelectedEffect():Void {
		//logger.error("terminating selected effect on "  + this);
		mcSelectHighlight._visible = false;
		if (! useSelectedEffect) return;
		selectedEffect.terminate();
	}
	
	public static function getSelectedItem(menuId:Number):MenuItemBorder {
		//logger.error("getSelectedItem " + selectedItem[menuId]);
		return selectedItem[menuId];
	}
	
	function selected():Void {
		dispatchEvent(EVENT_CLICKED, this);
		var menuItem:FlowMenuItem = FlowMenuItem(MovieClip(this).item); 		
		if (menuItem.isSelectable) {
			selectedItem[menuId].terminateSelectedEffect();
			selectedItem[menuId] = this;
			mcSelectHighlight._visible = true;
		}
		if (! useSelectedEffect) return;
		selectedEffect.execute(400);
	}
}