﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowplayer.Util;
import org.flowplayer.thumbs.Thumbnail;

class org.flowplayer.playlist.Clip {
 	public static var TYPES_ALL:Array = [ "flv", "jpg", "png", "mp3", "swf", "swf" ];
	private var id:String;
	private var keywords:Array;
	private var name:String;
	private var baseUrl:String;
	private var fileName:String;
	private var type:String;
	private var start:Number;
	private var end:Number;
	private var durationFromMetaData:Number;
	private var protected:Boolean;
	private var children:Array;
	private var controlEnabled:Boolean;
	private var linkUrl:String;
	private var linkWindow:String;
	private var resizeAllowed:Boolean;
	private var overlayFilename:String;
	private var overlayId:String;
	private var live:Boolean;
	private var showOnLoadBegin:Boolean;
	private var playCount:Number;
	private var maxPlayCount:Number;
	private var info:Object;
	private var thumb:Thumbnail;
	private var suggestionsInfoUrl:String;
	private var suggestionClips:Array;
	private var loadingSuggestions:Boolean;
	private var ads:Array;
	
	public static function createFromObject(baseUrl:String, clipObj:Object):Clip {
		if (clipObj.duration != undefined) {
			clipObj.start = 0;
			clipObj.end = clipObj.duration;
		}
		var clip:Clip = new Clip(clipObj.name, baseUrl, clipObj.url, clipObj.start, clipObj.end, clipObj.protected, 
			clipObj.controlEnabled, clipObj.linkUrl, clipObj.linkWindow, clipObj.type, clipObj.allowResize, 
			clipObj.overlay, clipObj.overlayId, clipObj.live, clipObj.showOnLoadBegin, clipObj.maxPlayCount,
		    clipObj.info, clipObj.thumbnailUrl, clipObj.suggestedClipsInfoUrl, clipObj.id, clipObj.keywords);
		return clip;
	}
	
	public static function createSuggestionInfo(baseUrl:String, suggestedClipsInfoUrl:String):Clip {
		var clip:Clip = new Clip();
		clip.suggestionsInfoUrl = Util.addBaseURL(baseUrl, suggestedClipsInfoUrl);
		clip.type = 'suggestions';
		return clip;
	}
	
	public static function createSimple(baseUrl:String, fileName:String, name:String):Clip {
		return new Clip(name, baseUrl, fileName);
	}
		
	public static function createSimpleWithThumbAndInfo(baseUrl:String, fileName:String, name:String, thumbUrl:String, info:Object, duration:Number):Clip {
		var clip:Clip = Clip.createSimple(baseUrl, fileName, name);
		clip.thumb = new Thumbnail(Util.addBaseURL(baseUrl, thumbUrl), duration);
		clip.info = info;
		return clip;
	}
	
	
//	public static function selection(name:String, baseUrl:String, fileName:String, start: Number, end:Number, protected:Boolean, enableControl:Boolean, linkUrl:String, linkWindow:String, type:String, allowResize:Boolean):Clip {
//		return new Clip(name, baseUrl, fileName, start, end, protected, enableControl, linkUrl, linkWindow, type, allowResize);
//	}
//	
//	public static function fromStart(name:String, baseUrl:String, fileName:String, duration:Number, protected:Boolean, enableControl:Boolean, linkUrl:String, linkWindow:String, type:String, allowResize:Boolean):Clip {
//		return new Clip(name, baseUrl, fileName, 0, duration, protected, enableControl, linkUrl, linkWindow, type, allowResize);
//	}
	
	private function Clip(name:String, baseUrl:String, fileName:String, start:Number, end:Number, protected:Boolean, 
		enableControl:Boolean, linkUrl:String, linkWindow:String, type:String, allowResize:Boolean, 
		overlayFileName:String, overlayId:String, live:Boolean, showOnLoadBegin:Boolean, maxPlayCount:Number, 
		info:Object, thumbnailUrl:String, suggestedClipsInfoUrl:String, id:String, keywords:Array) {

		if (Util.isCompleteURLWithProtocol(fileName)) {
			var lastSlashIndex:Number = fileName.lastIndexOf("/");
			this.baseUrl = fileName.substring(0, lastSlashIndex);
			this.fileName = fileName.substring(lastSlashIndex + 1);
		} else {
			this.baseUrl = baseUrl;
			this.fileName = fileName;
		}

		this.name = name;
		this.start = start;
		this.end = end;
		this.type = type || getTypeFor(fileName, overlayId);
		this.protected = protected;
		this.children = new Array();
		this.controlEnabled = enableControl == undefined || enableControl;
		this.linkUrl = linkUrl;
		this.linkWindow = linkWindow || "_blank";
		this.resizeAllowed = allowResize == undefined || allowResize;
		this.overlayFilename = overlayFileName;
		this.overlayId = overlayId;
		this.live = live;
		this.showOnLoadBegin = showOnLoadBegin == undefined || showOnLoadBegin;
		this.playCount = 0;
		this.maxPlayCount = maxPlayCount;
		this.info = info;
		if (thumbnailUrl){
			this.thumb = new Thumbnail(Util.addBaseURL(baseUrl, thumbnailUrl), getDuration());
		}
		this.suggestionsInfoUrl = Util.addBaseURL(baseUrl, suggestedClipsInfoUrl);
		this.id = id;
		this.keywords = keywords;
	}
	
	private static function getTypeFor(url:String, overlayId:String):String {
		if (url.toLowerCase().indexOf(".flv") > 0) return 'flv';
		if (url.toLowerCase().indexOf(".jpg") > 0) return 'jpg';
		if (url.toLowerCase().indexOf(".png") > 0) return 'png';
		if (url.toLowerCase().indexOf(".mp3") > 0) return 'mp3';
		if (url.toLowerCase().indexOf(".swf") > 0) return 'swf';
		if (overlayId) return "overlay";
		return "flv"; 
	}
	
	public function addChild(clip:Clip):Void {
		children.push(clip);
	}
	
	public function getChildren():Array {
		return (new Array()).concat(children);
	}
	
	public function getName():String {
		return name;
	}
	
	public function getURL(protectionCode:String):String {
		return Util.addBaseURL(baseUrl, getFilename(protectionCode));
	}
	
	public function getBaseURL():String {
		return baseUrl;
	}
	
	public function getFilename(protectionCode:String):String {
		if (protectionCode) {		
			var newLineIndex:Number = protectionCode.indexOf("\n");
			var code:String = newLineIndex > 0 ? protectionCode.substring(0, newLineIndex) : protectionCode;
			return code + this.fileName;
		}else {
			return fileName;
		}
	}
	
	public function getTotalDuration():Number {
		var duration:Number = 0; 
		for (var i : Number = 0; i < children.length; i++) {
			duration += children[i].getDuration();
		}
		return duration + getDuration();
	}

	public function getDuration():Number {
		return getEndTime() - getStartTime() || Util.checkForNonNumbers(durationFromMetaData);
	}
	
	public function getDurationFromMetaData():Number {
		return durationFromMetaData;
	}

	public function getStartTime():Number {
		return Util.checkForNonNumbers(start);
	}
	
	public function getEndTime():Number {
		return Util.checkForNonNumbers(end);
	}
	
	public function getType():String {
		return this.type;
	}
	
	public function isImage():Boolean {
		return this.type == 'png' || this.type == 'jpg' || this.type == 'overlay';
	}
	
	public function isVideo():Boolean {
		return this.type == 'flv';
	}
			
	public function isAudio():Boolean {
		return this.type == 'mp3';	
	}
	
	public function setType(type:String):Void {
		this.type = type;
	}
	
	public function toString():String {
		return "Clip: baseUrl " + baseUrl + ", fileName " + fileName + ", start " + start + ", end " + end + ", type " + type;
	}
	
	public function isProtected():Boolean {
		return this.protected;
	}
	
	public function isControlEnabled():Boolean {
		return this.controlEnabled;
	}
	
	public function getLinkURL():String {
		return this.linkUrl;
	}
	
	public function getLinkWindow():String {
		return this.linkWindow;
	}
	
	public function allowResize():Boolean {
		return this.resizeAllowed;	
	}

	public function getOverlayFileName():String {
		return this.overlayFilename;
	}
	
	public function getOverlayId():String {
		return this.overlayId;
	}
	
	public function isLive():Boolean {
		return this.live;
	}
	
	public function isShowOnLoadBegin():Boolean {
		return this.showOnLoadBegin;
	}
	
	public function getPlayCount():Number {
		return this.playCount;
	}
	
	public function played():Void {
		playCount++;
	}
	
	public function getMaxPlayCount():Number {
		return this.maxPlayCount;
	}
	
	public function maxPlayCountReached():Boolean {
		if (! maxPlayCount) return false;
		return playCount >= maxPlayCount;
	}
	
	public function getInfo():Object {
		return this.info;
	}

	public function getThumbnail():Thumbnail {
		return this.thumb;
	}

	public function getSuggestionsInfoUrl():String {
		return this.suggestionsInfoUrl;
	}
	
	public function setSuggestionClips(clips:Array):Void {
		this.suggestionClips = clips;
	}
	
	public function getSuggestionClips():Array {
		return this.suggestionClips;
	}

	public function isLoadingSuggestions():Boolean {
		return loadingSuggestions;
	}
	
	public function setIsLoadingSuggestions(isLoading:Boolean):Void {
		this.loadingSuggestions = isLoading;
	}
	
	public function getId():String {
		return this.id;
	}
	
	public function getKeywords():Array {
		return this.keywords;
	}
	
	public function setDurationFromMetaData(duration:Number):Void {
		this.durationFromMetaData = duration;
	}

	public function getAds():Array {
		return this.ads;
	}
	
	public function setAds(ads:Array):Void {
		this.ads = ads;
	}
	
	public function isOverlay():Boolean {
		return type == 'overlay';	
	}

}