/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import cinqetdemi.JSONConfig;
import LuminicBox.Log.Logger;
import org.flowlib.Observable;
import org.flowplayer.thumbs.Thumbnail;
import org.flowplayer.playlist.Clip;
import org.flowplayer.playlist.PlayList;
import LuminicBox.Log.ConsolePublisher;
/**
 * @author Anssi
 */
class org.flowplayer.thumbs.SuggestionInfoLoader extends Observable {
	
	public static var EVENT_SUGGESTIONS_LOADED:String = "onSuggestionInfoLoaded";
	public static var EVENT_SUGGESTIONS_LOAD_ERROR:String = "onSuggestionInfoLoadError";
	private var loader:JSONConfig;
	private var logger:Logger;
	private var clip:Clip;

	
	public function SuggestionInfoLoader() {
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
		loader = new JSONConfig();
		loader.addConfigLoadListener(this);
		loader.addLoadErrorListener(this);
	}

	public function loadForClip(clip:Clip):Void {
		if (! clip.getSuggestionsInfoUrl())
			return;
		load(clip.getSuggestionsInfoUrl(), clip);
	}
	
	public function load(url:String, clip:Clip):Void {
		if (clip.isLoadingSuggestions() || clip.getSuggestionClips()) return;
		clip.setIsLoadingSuggestions(true);
		loader.fileName = url;
		this.clip = clip;
		loader.loadConfig();
	}
	
	public function onConfigLoadError():Void {
		logger.error("Failed to load suggestions, url '" + clip.getSuggestionsInfoUrl() + "'");
		clip.setIsLoadingSuggestions(false);
//		dispatchEvent(EVENT_SUGGESTIONS_LOAD_ERROR);
	}
	
	public function onConfigLoaded():Void {		
		clip.setIsLoadingSuggestions(false);
		logger.info("Received suggestion info from " + clip.getSuggestionsInfoUrl());
		if (clip) {
			clip.setSuggestionClips(buildSuggestedClipsArray(loader.getConfigObject()));
			dispatchEventWithObj(EVENT_SUGGESTIONS_LOADED, clip);
		}
		else
			dispatchEventWithObj(EVENT_SUGGESTIONS_LOADED, buildSuggestedClipsArray(loader.getConfigObject()));
	}
	
	function buildSuggestedClipsArray(infoObj:Object):Array {
		return PlayList.createClips(infoObj.baseURL, infoObj.clips, undefined, undefined, false, false);
	}

	private function createThumbnail(clipObj:Object):Object {
		var clip:Clip;
		return new Thumbnail(clipObj.url, undefined, clip);
	}

}