/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.skin.Skin;

/**
 * @author Anssi
 */
class org.flowplayer.thumbs.SuggestionInfoView extends NewAbstractMovieClip {

	private var nameTxt:TextField;
	private var infoTxt:TextField;
	var skin:Skin;
	
	public function SuggestionInfoView() {
		super();
	}

	private function createChildren():Void {
		createNameTextField();
		createInfoTextField();
	}
	
	function arrange():Void {
		if (! clipsLoaded([ nameTxt, infoTxt ]))
			return;
		nameTxt._x = 0;
		nameTxt._y = 0;
		
		infoTxt._x = 0;
		infoTxt._y = nameTxt._y + nameTxt._height + 5;
	}
	
	public function setData(name:String, info:Object):Void {
		if (! (infoTxt && nameTxt))
			logger.error("setData: text fields are not created yet, cannot set data");
			
		nameTxt.text = name || "";
		
		infoTxt.text = "";
		var numProps:Number = 0;
		for (var prop:String in info) {
			var infoItem:String = info[prop];
			infoTxt.text += prop + ": " + infoItem + "\n";
			numProps++;
		}
		infoTxt._height = numProps * 20;
		// strip out last "\n"
		infoTxt.text = infoTxt.text.substring(0, infoTxt.length - 1);
		
		logger.debug("name set to " + nameTxt.text + " in " + nameTxt);
		logger.debug("info set to " + infoTxt.text + " in " + infoTxt);
	}
	
	private function createNameTextField():Void {
		createTextField("nameTxt", getNextHighestDepth(), 0, 0, 120, 40);
		initTextField(nameTxt, 12, true);
		nameTxt.wordWrap = true;
		nameTxt.multiline = false;
		nameTxt.selectable = false;
		nameTxt.textColor = 0xFF0000;
	}
	
	private function createInfoTextField():Void {
		createTextField("infoTxt", getNextHighestDepth(), 0, 0, 200, 100);
		initTextField(infoTxt, 12);
		infoTxt.wordWrap = true;
		infoTxt.multiline = true;
		infoTxt.selectable = false;
		infoTxt.textColor = 0xffffff;
	}
	
	public function initTextField(field:TextField, size:Number, bold:Boolean):Void {
		var format:TextFormat = getSkin().getDefaultTextFormat(12);
//		field.antiAliasType = "advanced";
		format.align = "left";
		format.bold = bold;
		field.setNewTextFormat(format);
	}
}