/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.ClipDescriptor;
import org.flowplayer.thumbs.ViewPortThumbStrip;
import org.flowplayer.thumbs.SuggestionInfoView;
import org.flowplayer.thumbs.Thumbnail;
import org.flowplayer.Util;
import org.flowplayer.playlist.Clip;
import org.flowplayer.MediaView;
import org.flowplayer.thumbs.ThumbStrip;

/**
 * @author Anssi
 */
class org.flowplayer.thumbs.SuggestionView extends NewAbstractMovieClip implements MediaView {
	
	public static var EVENT_REPLAY_CKICKED:String = "onReplayClicked";
	private var holder:ViewPortThumbStrip;
	private var holder2:ViewPortThumbStrip;
	private var infoView1:SuggestionInfoView;
	private var infoView2:SuggestionInfoView;
	private var btnRight:MovieClip;
	private var btnLeft:MovieClip;
	private var mask:MovieClip;
	private var mcBorder:MovieClip;
	private var btnReplay:MovieClip;
	private var currentPos:Number;
	private var thumbs:Array;
	private var clips:Array;
	private var THUMBS_PER_VIEW:Number = 2;
	private var holdersArranged:Number;


	
	public function SuggestionView() {
		super();
//		this.visible = false;
		this.currentPos = 1;
		ease.easeId = -1;
		ease.view = this;
	}
	
	public function setClips(clips:Array):Void {
		logger.info("ThumbList.setThumbnails");
		logger.debug(clips);
		this.currentPos = 1;
		this.clips = clips;
		this.thumbs = extractThumbs(clips);
		holdersArranged = 0;
		holder.setThumbnails(thumbs);
		holder2.setThumbnails(thumbs);
	}
		
	private function createChildren():Void {
		mcBorder = createBorder(this, 300, 10);
		btnReplay = getSkin().createReplayButton(this);
		
		btnReplay.onRelease = function() {
			MovieClip(this)._parent.replayClicked();
		};

		btnLeft = getSkin().createBackwardButton(this);
		btnRight = getSkin().createForwardButton(this);
		
		btnLeft.onRelease = function() {
			MovieClip(this)._parent.reverseClips();
		};
		
		btnRight.onRelease = function() {
			MovieClip(this)._parent.advanceClips();
		};

		mask = createMask();
		this.setMask(mask);
	}
		
	private function createMask():MovieClip {
		var mask:MovieClip = createEmptyMovieClip("mask", getNextHighestDepth());
		mask.beginFill(0xFFFFFF, 100);
		drawRectangle(mask, 0, 0, 100, 20);
		mask.endFill();
		return mask;
	}
	
	public function createBorder(parent:NewAbstractMovieClip, width:Number, height:Number):MovieClip {
		var clip:MovieClip = parent.createEmptyMovieClip("mcBorder", parent.getNextHighestDepth());
		clip.beginFill(0x000000, 50);
		Util.drawRectangleWithBorder(clip, 0, 0, width, height);
		clip.endFill();
		return clip;
	}
	
	private function getChildDescriptors():Array {
		return [ 
			new ClipDescriptor("ViewPortThumbStrip", "holder", null),
			new ClipDescriptor("ViewPortThumbStrip", "holder2", null),
			new ClipDescriptor("SuggestionInfoView", "infoView1", null),
			new ClipDescriptor("SuggestionInfoView", "infoView2", null)
		];
	}
	
	private function allChildrenCreated(clips:Array):Void {
		logger.info("SuggestionView.allChildrenCreated");
		this.holder = clips[0];
		holder.addEventListener("onArranged", this);
		this.holder2 = clips[1];
		holder2.addEventListener("onArranged", this);
		
//		this.btnLeft = clips[2];
//		btnLeft.onRelease = function() {
//			MovieClip(this)._parent.reverseClips();
//		};
//		
//		this.btnRight = clips[3];
//		btnRight.onRelease = function() {
//			MovieClip(this)._parent.advanceClips();
//		};
		
		this.infoView1 = clips[2];
		this.infoView2 = clips[3];
	}
	
	function arrange():Void {
		
		btnReplay._y = 10;
		Util.centerClip(btnReplay, __width);
		
		holder._y = btnReplay._y + btnReplay._height + 10;
		holder._x = 0;
		holder.setViewPortWidth(__width);
		holder2._x = this.__width;
		holder2._y = holder._y;
		holder2.setViewPortWidth(__width);
		
		infoView1._x = holder.getThumbSpacing();
		infoView1._y = holder._y + holder._height + 10;
		infoView2._x = infoView1._x * 2 + holder.getThumbWidth();
		infoView2._y = infoView1._y;
		setThumbInfos();
		
		var yPos:Number = infoView1._y + infoView1._height + 5;
		mcBorder._y = 0;
		mcBorder._width = __width;
		
		if (__width > holder._width) {
			btnRight._visible = false;
			btnLeft._visible = false;
			mcBorder._height = infoView1._y + infoView1._height + 10;
		} else {
//			Util.centerClip(holder, __width);
			btnRight._visible = true;
			btnLeft._visible = true;
			btnLeft._x = 10;
			btnLeft._y = yPos;
			btnRight._x = __width - btnRight._height - 10;
			btnRight._y = btnLeft._y;
			mcBorder._height = btnRight._y + btnRight._height + 10;
		}
		
		mask._width = __width;
		mask._height = this._height;
//		this.__height = mcBorder._height;
	}
	
	private function setThumbInfos():Void {
		infoView1.setData(getClip1().getName(), getClip1().getInfo());
		infoView2.setData(getClip2().getName(), getClip2().getInfo());
	}
	
	private function getClip1():Clip {
		return clips[currentPos-1];
	}
	
	private function getClip2():Clip {
		var pos:Number = currentPos == thumbs.length ? 0 : currentPos;
		return clips[pos];
	}
	
	function onArranged():Void {
		logger.debug("SuggestionView received onArranged()");
		holdersArranged++;
		if (holdersArranged == 2) {
			arrange();
			logger.debug("SuggestionView dispatching onArranged since both holders were arranged");
			dispatchEvent("onArranged", this);
		}
	}
	
	function advanceClips():Void {
		if (ease.easeId >= 0) return;
		
		if (! thumbsDivideToScreenfulls() && currentPos == thumbs.length - THUMBS_PER_VIEW) {
			logger.debug("sliding in holder2 to fill in lonely space");
			currentPos += THUMBS_PER_VIEW;
			holder2._x = __width + holder.getThumbWidth() + holder.getThumbSpacing();
			startAdvanceEase([ holder, holder2], this.__width, true);
			setThumbInfos();
			return;
		}
		
		
		if (! thumbsDivideToScreenfulls() && currentPos == thumbs.length) {
			logger.debug("moving only division left-out width");
			currentPos = 1;
			holder2._x = holder._x + holder._width + holder.getThumbSpacing();
			startAdvanceEase([ holder, holder2], spaceLeftForHolder2AfterDivision(), false);
			setThumbInfos();
			return;
		}
		
		if (currentPos == thumbs.length - THUMBS_PER_VIEW + 1) {
			currentPos = 1;
			holder2._x = __width;
			startAdvanceEase([ holder, holder2], this.__width, true);
			setThumbInfos();
			return;
		}

		currentPos += THUMBS_PER_VIEW;
		startAdvanceEase([ holder ]);
		setThumbInfos();
	}
	
	function reverseClips():Void {
		if (ease.easeId >= 0) return;
		
		if (! thumbsDivideToScreenfulls() && currentPos == 1) {
			currentPos = thumbs.length;
			holder2._x = - holder2._width - holder2.getThumbSpacing();
			startReverseEase([ holder, holder2], spaceLeftForHolder2AfterDivision(), false);
			setThumbInfos();
			return;
		}
		
		if (! thumbsDivideToScreenfulls() && currentPos == thumbs.length) {
			currentPos -= THUMBS_PER_VIEW;
			holder2._x = holder._x - holder2._width - holder2.getThumbSpacing();
			startReverseEase([ holder, holder2], this.__width, true);
			setThumbInfos();
			return;
		}
		
		if (currentPos == 1) {
			currentPos = thumbs.length - THUMBS_PER_VIEW + 1;
			holder2._x = - holder2._width - THUMBS_PER_VIEW * holder2.getThumbSpacing();
			startReverseEase([ holder, holder2], this.__width, true);
			setThumbInfos();
			return;
		}
		
		currentPos -= THUMBS_PER_VIEW;	
		startReverseEase([ holder ]);
		setThumbInfos();
	}
	
	private function spaceLeftForHolder2AfterDivision():Number {
		return thumbs.length % THUMBS_PER_VIEW * (holder.getThumbWidth() + holder.getThumbSpacing());
	}
	
	private function thumbsDivideToScreenfulls():Boolean {
		return thumbs.length % THUMBS_PER_VIEW == 0;
	}
	
	function startAdvanceEase(holders:Array, easeDistance:Number, switchHolders:Boolean):Void {
		startEase(holders, - easeDistance || - this.__width, holders.length == 2 && switchHolders);
	}
	
	function startReverseEase(holders:Array, easeDistance:Number, switchHolders:Boolean):Void {
		startEase(holders, easeDistance || this.__width, holders.length == 2 && switchHolders);
	}
	
	function startEase(holders:Array, distance:Number, switchHolders:Boolean):Void {
		for (var i:Number = 0; i < holders.length; i++) {
			var clip:MovieClip = holders[i];
			clip.target = clip._x + distance;
		}
		ease.easeId = setInterval(this, "ease", 10, holders, 0.2, switchHolders);
	}
	
	function ease(clips:Array, easing:Number, switchHolders:Boolean):Void {
		var clipsInTarget:Boolean = moveClips(clips, easing);

		if (clipsInTarget) {
			clearInterval(arguments.callee.easeId);
			arguments.callee.easeId = -1;
			NewAbstractMovieClip.logger.debug("clips easing done, clearing interval");
			
			if (switchHolders)
				arguments.callee.view.switchHolders();
		}
	}

	private function moveClips(clips:Array, easing:Number):Boolean {
		var numClipsInTarget:Number = 0;
		for (var i:Number = 0; i < clips.length; i++) {
			var clip:MovieClip = clips[i];
			if (clip) {
				var vx:Number = (clip.target - clip._x) * easing;
				clip._x += vx;
	
				var dx:Number = clip.target - clip._x;
				if (Math.abs(dx) < 1) {
					numClipsInTarget++;
					clip._x = clip.target;
				}
			} else {
				numClipsInTarget++;
			}
		}
		return numClipsInTarget == clips.length;
	}

	function switchHolders():Void {
		var temp:ViewPortThumbStrip = holder;
		holder = holder2;
		holder2 = temp;
		NewAbstractMovieClip.logger.debug("switched holders, holder is now " + holder);
	}
	
	private function extractThumbs(clips:Array):Array {
		var thumbs:Array = new Array();
		for (var i:Number = 0; i < clips.length; i++) {
			var clip:Clip = clips[i];
			var thumb:Thumbnail = clip.getThumbnail();
			if (thumb) {
				Object(thumb).suggestionViewArrayIndex = i;
				thumbs.push(clip.getThumbnail());
			}
		}
		return thumbs;
	}

	function handlesOnRelease():Boolean {
		return true;
	}
	
	public function onThumbClicked(thumb:Thumbnail):Void {
		// dispatch to my listeners
		dispatchEvent(ThumbStrip.THUMB_CLICKED_EVENT, clips[Object(thumb).suggestionViewArrayIndex]);
	}

	public function replayClicked():Void {
		dispatchEvent(SuggestionView.EVENT_REPLAY_CKICKED);
	}
}