/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.AlphaCycleEffect;
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.ClipDescriptor;
import org.flowplayer.FlowText;
import org.flowplayer.thumbs.Thumbnail;
import org.flowplayer.thumbs.ThumbStrip;

/**
 * @author anssi
 */
class org.flowplayer.thumbs.ThumbnailView extends NewAbstractMovieClip {

	private var BORDER_WIDTH = 3;
	private var thumb:MovieClip;
	private var model:Thumbnail;
	private var mcHighlight:MovieClip;
	private var mcTimeLabel:FlowText;
	private var clickEffect:AlphaCycleEffect;
	private var highlightEffect:AlphaCycleEffect;
//
//	private function getChildDescriptors():Array {
//		return [ new ClipDescriptor("FlowText", "mcTimeLabel", null) ];
//	}

	public function ThumbnailView() {
		super();
		this.visible = false;
	}

	public function loadThumb(thumbNail:Thumbnail):Void {
		logger.info(this._name + ", loadThumb " + thumbNail.getUrl());
		this.model = thumbNail;
		loadClips([ new ClipDescriptor(null, "thumb", thumbNail.getUrl()) ]);
	}

	private function createChildren():Void {
		mcHighlight = createEmptyMovieClip("mcHighLight", getNextHighestDepth());
		mcHighlight._visible = false;
	}

	private function loadInit(clip:MovieClip):Void {
		this.thumb = clip;
		
		this.thumb.onRelease = function():Void {
			this._parent.thumbClicked();
		};
		clickEffect = new AlphaCycleEffect(thumb);
		highlightEffect = new AlphaCycleEffect(mcHighlight);
	}
	

	function arrange():Void {
		if (model == undefined) return;
		drawRectangle(this, 0, 0, thumb._width + BORDER_WIDTH * 2, thumb._height + BORDER_WIDTH * 2, false);
		
		if (isLoaded(thumb) && mcTimeLabel == undefined) {
			mcTimeLabel = FlowText(attachMovie("FlowText", "mcTimeLabel", getNextHighestDepth() + 10, null));
			mcTimeLabel.drawBorderGradient = true;
			mcTimeLabel.drawBorder();
			mcTimeLabel.setTextProperties("left", 0x000000, 10);
			mcTimeLabel.text = model.getTimeFormatted();
		}

		thumb._x = BORDER_WIDTH;
		thumb._y = BORDER_WIDTH;
		
		if (mcTimeLabel != undefined) {
			mcTimeLabel._x = BORDER_WIDTH;
			mcTimeLabel._y = this._height - mcTimeLabel._height;
		}
		
		dispatchEvent("onArranged", this);
	}
	
	private function fillBorder():Void {
		this.clear();
		this.beginFill(0x113311, 100);
		drawRectangle(this, 0, 0, thumb._width + BORDER_WIDTH * 2, thumb._height + BORDER_WIDTH * 2, false);
		this.endFill();
		
		mcHighlight.clear();
		mcHighlight.beginFill(0x33FF33, 50);
		drawRectangle(mcHighlight, 0, 0, thumb._width + BORDER_WIDTH * 2, thumb._height + BORDER_WIDTH * 2, false);
		mcHighlight.endFill();
	}
	
	private function thumbClicked():Void {
		if (! isMouseOverClip(thumb)) return;
		clickEffect.executeOneCycle();
		logger.info(this._name + " clicked");
		dispatchEvent(ThumbStrip.THUMB_CLICKED_EVENT, model);
	}
	
	public function startHighlight():Void {
		logger.debug("startHightlight on " + this._name);
		mcHighlight._visible = true;
		fillBorder();
		highlightEffect.execute(100);
	}
	
	public function stopHighlight():Void {
		highlightEffect.terminate();
		this.clear();
		mcHighlight._visible = false;
	}
}