/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowplayer.ads.InVideoAd;
import org.flowplayer.ads.InVideoAdController;
import org.flowplayer.ads.InVideoAdView;
import org.flowplayer.config.HierarchicalConfig;
import org.flowplayer.playlist.Clip;
import org.flowplayer.skin.Skin;

import asunit.framework.TestCase;

import LuminicBox.Log.ConsolePublisher;
import org.flowplayer.ads.AdServerClient;

class org.flowplayer.ads.AdServerClientTest extends TestCase {
	
	private var adServerUrl:String = "http://adserver.flowplayer.org";
	private var adServerAuth:String = "foo";
	private var client : AdServerClient;


	public function setUp():Void {
		var config:HierarchicalConfig = new HierarchicalConfig("{ adServerUrl: '"+adServerUrl+"', adServerAuth: '"+adServerAuth+"' }", "test config");
		client = new AdServerClient(config);
	}
		
	public function AdServerClientTest(testMethod : String) {
		super(testMethod);
	}
	
	public function testAdsUrlForUncategorizedClip():Void {
		var clip:Clip = Clip.createFromObject("http://flowplayer.org", { url: 'coolstuff.flv', duration: 20 });
		var result:String = client.adsUrlFor(clip);
		var expected:String = adServerUrl + "/ads?s=" + adServerAuth + "&amp;videoURL=" + clip.getURL() + "&amp;debug=true&amp;duration=" + clip.getDuration();
		assertEquals("result was " + result + ", expecting " + expected, expected, result);
	}
	
	public function testAdsUrlWithKeywords():Void {
		var clip:Clip = Clip.createFromObject("http://flowplayer.org", { url: 'coolstuff.flv', duration: 20, keywords: [ 'food', 'drink', "candy" ], id: 'testad1' });
		var result:String = client.adsUrlFor(clip);
		var expected:String = adServerUrl + "/ads?s=" + adServerAuth + "&amp;videoURL=" + clip.getURL() + "&amp;videoKey=testad1" +
			"&amp;debug=true&amp;duration=" + clip.getDuration() + "&amp;keyword=food,drink,candy";
		assertEquals("result was " + result + ", expecting " + expected, expected, result);
	}

}