﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import asunit.framework.TestCase;
import org.flowplayer.menu.FlowMenu;
import org.flowplayer.menu.FlowMenuItem;
import org.flowplayer.menu.MenuItemBorder;

/**
 * @author anssi
 */
class org.flowplayer.menu.FlowMenuTest extends TestCase {
	
	private var menu:FlowMenu;

	private var selectedItem : FlowMenuItem;
	
	public function FlowMenuTest(name:String) {
		super(name);
	}
	
	public function setUp():Void {
		menu = FlowMenu(attachMovie("FlowMenu"));
	}
	
	public function tearDown():Void {
//		menu.removeMovieClip();
	}
	
	public function testInstancCreated():Void {
		assertTrue("instance was " + menu, menu instanceof FlowMenu);
	}
	
	public function testAddItem():Void {
		var name = "item 1";
		var menuItem:FlowMenuItem = menu.addSelectableItem(name, 1);
		assertNotUndefined("menu item was not defined", menuItem);
		assertEquals("Returned " + menuItem.name, name, menuItem.name);
		assertEquals(1, menuItem.object);
	}
	
	public function testAddTwoItems():Void {
		var item1:FlowMenuItem = menu.addSelectableItem("item1", 1);
		var item2:FlowMenuItem = menu.addSelectableItem("item2", 2);
		var items:Array = menu.getItems();
		
		assertTrue("was not item1 " + items[0], 1 == item1.object);
		assertTrue("was not item2 " + items[1], 2 == item2.object);
	}
	
	public function testShow():Void {
		var item1:FlowMenuItem = menu.addSelectableItem("Fit to window", 1);
		var item2:FlowMenuItem = menu.addSelectableItem("Orig. size", 2);

		var menuItemClips:Array = menu.show();

		verifyClip("clip1", menuItemClips[0], item1);
		verifyClip("clip2", menuItemClips[1], item2);
	}
	
	private function verifyClip(name:String, clip:MovieClip, item:FlowMenuItem) {
		assertNotUndefined(name + " was undefined", clip);
		assertNotUndefined(name + " did not have a textField", clip.text);
		assertEquals(name + "'s label was not " + item.name + " but was " + clip.text.text, item.name, clip.text.text);		
	}
	
	public function testOnSelected():Void {
		var item1:FlowMenuItem = menu.addSelectableItem("item1", 1);
		var item2:FlowMenuItem = menu.addSelectableItem("item2", 2);

		var menuItemClips:Array = menu.show();
		var clip:MenuItemBorder = menuItemClips[1];
		
		menu.addEventListener(FlowMenu.EVENT_SELECTED, this);
		clip.selected();
		assertEquals("did not select item2", 2, this.selectedItem.object);
	}
	
	public function onMenuSelected(menuItem:FlowMenuItem):Void {
		this.selectedItem = menuItem;
	}
	
	public function testAddItemSelected():Void {
		menu.addEventListener(FlowMenu.EVENT_SELECTED, this);

		var item1:FlowMenuItem = menu.addSelectableItem("item1", 1, true);
		var item2:FlowMenuItem = menu.addSelectableItem("item2", 2);
		

		var menuItemClips:Array = menu.show();

		assertEquals("did not select item1 when the menu was shown", 1, this.selectedItem.object);
		
		// now select second
		var clip2:MenuItemBorder = menuItemClips[1];
		clip2.selected();
		assertEquals("did not change selection", 2, this.selectedItem.object);
	}
}