/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import asunit.framework.TestCase;
import org.flowplayer.thumbs.SuggestionInfoLoader;
import org.flowplayer.thumbs.Thumbnail;
import org.flowplayer.playlist.Clip;
import LuminicBox.Log.Logger;
import LuminicBox.Log.ConsolePublisher;

class org.flowplayer.thumbs.SuggestionInfoLoaderTest extends TestCase {
	
	private var logger:Logger;
	
	public function SuggestionInfoLoaderTest(testMethod:String) {
		super(testMethod);
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
	}

	public function testBuildSuggestionInfo():Void {
		var loader:SuggestionInfoLoader = new SuggestionInfoLoader();
		var suggestedClips:Array = loader.buildSuggestedClipsArray(createInfoObj());
		for (var i:Number = 0; i < suggestedClips.length; i++) {
			var clip:Clip = suggestedClips[i];
			var j:Number = i + 1;
			assertEquals('clip url: ' + clip.getURL(), 'http://foobar/url' + j, clip.getURL());
			assertEquals('getRelatedClipsInfoUrl: ' + clip.getSuggestionsInfoUrl(), 'http://foobar/relatedUrl' + j, clip.getSuggestionsInfoUrl());
			assertEquals('duration', 180, clip.getDuration());
			assertEquals('thumb url', 'http://foobar/thumbUrl' + j, clip.getThumbnail().getUrl());
			assertEquals('viewed', 631, clip.getInfo().viewed);
			assertEquals('category', 'scheisse' + j, clip.getInfo().category);
//			assertEquals('thumbUrl1', thumb)
		}
	}
	
	public function testLoad():Void {
		var loader:SuggestionInfoLoader = new SuggestionInfoLoader();
		loader.addListener(SuggestionInfoLoader.EVENT_SUGGESTIONS_LOADED, this);
		loader.load("suggestions.js");
	}
	
	public function onSuggestionInfoLoaded(suggestion:Object):Void {
		logger.debug("received suggestions:");
		logger.debug(suggestion);
	}

	private function createInfoObj(): Object {
		return {
			baseURL: 'http://foobar', 
			clips: [
				{ url: 'url1', name: 'name1', suggestedClipsInfoUrl: 'relatedUrl1',
					duration: 180, thumbnailUrl: 'thumbUrl1',
					info: { viewed: 631, category: 'scheisse1' } },
				{ url: 'url2', name: 'name2', suggestedClipsInfoUrl: 'relatedUrl2',
					duration: 180, thumbnailUrl: 'thumbUrl2',
					info: { viewed: 631, category: 'scheisse2' } }
			]
		};
	}

}